@@ -1,27 +1,24 @@
 """Commands for the TurboGears command line tool."""
 
-import optparse
-import sys
-import os
-import os.path
 import glob
+import optparse
+import os
+import sys
 
 import pkg_resources
-import dispatch
+from peak.rules import NoApplicableMethods
 
-import configobj
-
-import turbogears
-from turbogears.util import get_model, load_project_config, \
-        get_project_config, get_package_name
+from turbogears import config, database, widgets
 from turbogears.identity import SecureObject, from_any_host
-from turbogears import config, database
+from turbogears.util import (get_model, get_project_name, get_project_config,
+    get_package_name, load_project_config)
+
 from sacommand import sacommand
 
 sys.path.insert(0, os.getcwd())
 
-no_connection_param = ["help", "list"]
-no_model_param = ["help"]
+no_connection_param = ['help', 'list']
+no_model_param = ['help']
 
 
 def silent_os_remove(fname):
@@ -50,8 +47,8 @@ class CommandWithDB(object):
         """Chooses the config file, trying to guess whether this is a
         development or installed project."""
         load_project_config(self.config)
-        self.dburi = config.get("sqlobject.dburi", None)
-        if self.dburi and self.dburi.startswith("notrans_"):
+        self.dburi = config.get('sqlobject.dburi', None)
+        if self.dburi and self.dburi.startswith('notrans_'):
             self.dburi = self.dburi[8:]
 
 
@@ -69,14 +66,13 @@ class SQL(CommandWithDB):
     need_project = True
 
     def __init__(self, version):
-        if len(sys.argv) == 1 or sys.argv[1][0] == "-":
+        if len(sys.argv) == 1 or sys.argv[1][0] == '-':
             parser = optparse.OptionParser(
-                usage="%prog sql [command]\n\n" \
-                      "hint: '%prog sql help' will list the sqlobject " \
-                      "commands",
-                version="%prog " + version)
-            parser.add_option("-c", "--config", help="config file",
-                              dest="config")
+                usage="%prog sql [command]\n\nhint: '%prog sql help'"
+                    " will list the sql specific commands",
+                version='%prog ' + version)
+            parser.add_option('-c', '--config', help="config file",
+                dest='config')
             options, args = parser.parse_args(sys.argv[1:3])
 
             if not options.config:
@@ -91,51 +87,52 @@ class SQL(CommandWithDB):
         self.find_config()
 
     def run(self):
-        """Executes the sqlobject-admin code."""
-        if not "--egg" in sys.argv and not turbogears.util.get_project_name():
-            print "This don't look like a TurboGears project."
+        """Run the sqlobject-admin tool or sacommand module functions."""
+
+        if not '--egg' in sys.argv and not get_project_name():
+            print "This doesn't look like a TurboGears project."
             return
         else:
             command = sys.argv[1]
 
-            if config.get("sqlalchemy.dburi"):
+            if config.get('sqlalchemy.dburi'):
                 try:
                     sacommand(command, sys.argv)
-                except dispatch.interfaces.NoApplicableMethods:
-                    sacommand("help", [])
+                except NoApplicableMethods:
+                    sacommand('help', [])
                 return
 
             try:
-                from sqlobject.manager import command
+                from sqlobject.manager import command as socommand
             except ImportError:
-                print "Error: SQLObject not installed."
+                from turbogears.util import missing_dependency_error
+                print missing_dependency_error('SQLObject')
                 return
 
-            sqlobjcommand = command
-            if sqlobjcommand not in no_connection_param:
+            if command not in no_connection_param:
                 if self.dburi:
                     print "Using database URI %s" % self.dburi
                     sys.argv.insert(2, self.dburi)
-                    sys.argv.insert(2, "-c")
+                    sys.argv.insert(2, '-c')
                 else:
                     print ("Database URI not specified in the config file"
                         " (%s).\nPlease be sure it's on the command line."
                             % (self.config or get_project_config()))
 
-            if sqlobjcommand not in no_model_param:
-                if not "--egg" in sys.argv:
-                    eggname = glob.glob("*.egg-info")
+            if command not in no_model_param:
+                if not '--egg' in sys.argv:
+                    eggname = glob.glob('*.egg-info')
                     if not eggname or not os.path.exists(
-                            os.path.join(eggname[0], "sqlobject.txt")):
+                            os.path.join(eggname[0], 'sqlobject.txt')):
                         eggname = self.fix_egginfo(eggname)
-                    eggname = eggname[0].replace(".egg-info", "")
-                    if not "." in sys.path:
-                        sys.path.append(".")
-                        pkg_resources.working_set.add_entry(".")
+                    eggname = eggname[0].replace('.egg-info', '')
+                    if not '.' in sys.path:
+                        sys.path.append('.')
+                        pkg_resources.working_set.add_entry('.')
                     sys.argv.insert(2, eggname)
-                    sys.argv.insert(2, "--egg")
+                    sys.argv.insert(2, '--egg')
 
-            command.the_runner.run(sys.argv)
+            socommand.the_runner.run(sys.argv)
 
     def fix_egginfo(self, eggname):
         """Add egg-info directory if necessary."""
@@ -149,18 +146,18 @@ I can fix this automatically. Would you like me to?
         dofix = raw_input("Enter [y] or n: ")
         if not dofix or dofix.lower()[0] == 'y':
             oldargs = sys.argv
-            sys.argv = ["setup.py", "egg_info"]
+            sys.argv = ['setup.py', 'egg_info']
             import imp
-            imp.load_module("setup", *imp.find_module("setup", ["."]))
+            imp.load_module('setup', *imp.find_module('setup', ['.']))
             sys.argv = oldargs
 
             import setuptools
             package = setuptools.find_packages()[0]
-            eggname = glob.glob("*.egg-info")
-            sqlobjectmeta = open(os.path.join(eggname[0], "sqlobject.txt"), "w")
-            sqlobjectmeta.write("""db_module=%(package)s.model
-history_dir=$base/%(package)s/sqlobject-history
-""" % dict(package=package))
+            eggname = glob.glob('*.egg-info')
+            sqlobjectmeta = open(os.path.join(eggname[0], 'sqlobject.txt'), 'w')
+            sqlobjectmeta.write('db_module=%(package)s.model\n'
+                'history_dir=$base/%(package)s/sqlobject-history'
+                % dict(package=package))
         else:
             sys.exit(0)
         return eggname
@@ -182,46 +179,58 @@ class Shell(CommandWithDB):
         """Run the shell"""
         self.find_config()
 
-        mod = get_model()
-        if mod:
-            locals = mod.__dict__
-        else:
-            locals = dict(__name__="tg-admin")
+        locals = dict(__name__='tg-admin')
+        try:
+            mod = get_model()
+            if mod:
+                locals.update(mod.__dict__)
+        except (pkg_resources.DistributionNotFound, ImportError), e:
+            mod = None
+            print "Warning: Failed to import your data model: %s" % e
+            print "You will not have access to your data model objects."
+            print
 
-        if config.get("sqlalchemy.dburi"):
+        if config.get('sqlalchemy.dburi'):
             using_sqlalchemy = True
-            database.get_engine()
-            locals.update(dict(session=database.session,
-                metadata=database.metadata))
+            database.bind_metadata()
+            locals.update(session=database.session,
+                metadata=database.metadata)
         else:
             using_sqlalchemy = False
 
+        class CustomShellMixin(object):
+            def commit_changes(self):
+                if mod:
+                    # XXX Can we check somehow, if there are actually any
+                    # database changes to be commited?
+                    r = raw_input("Do you wish to commit"
+                        " your database changes? [yes]")
+                    if not r.startswith('n'):
+                        if using_sqlalchemy:
+                            self.push("session.flush()")
+                        else:
+                            self.push("hub.commit()")
+
         try:
             # try to use IPython if possible
-            import IPython
+            from IPython import iplib, Shell
 
-            class CustomIPShell(IPython.iplib.InteractiveShell):
+            class CustomIPShell(iplib.InteractiveShell, CustomShellMixin):
                 def raw_input(self, *args, **kw):
                     try:
-                        return IPython.iplib.InteractiveShell.raw_input(self,
-                            *args, **kw) # needs decoding (see below)?
+                         # needs decoding (see below)?
+                        return iplib.InteractiveShell.raw_input(self, *args,
+                                                                **kw)
                     except EOFError:
-                        r = raw_input("Do you wish to commit your "
-                                    "database changes? [yes]")
-                        if not r.lower().startswith("n"):
-                            if using_sqlalchemy:
-                                self.push("session.flush()")
-                            else:
-                                self.push("hub.commit()")
+                        self.commit_changes()
                         raise EOFError
 
-            shell = IPython.Shell.IPShell(user_ns=locals,
-                                          shell_class=CustomIPShell)
+            shell = Shell.IPShell(user_ns=locals, shell_class=CustomIPShell)
             shell.mainloop()
         except ImportError:
             import code
 
-            class CustomShell(code.InteractiveConsole):
+            class CustomShell(code.InteractiveConsole, CustomShellMixin):
                 def raw_input(self, *args, **kw):
                     try:
                         import readline
@@ -239,13 +248,7 @@ class Shell(CommandWithDB):
                                     pass
                         return r
                     except EOFError:
-                        r = raw_input("Do you wish to commit your "
-                                      "database changes? [yes]")
-                        if not r.lower().startswith("n"):
-                            if using_sqlalchemy:
-                                self.push("session.flush()")
-                            else:
-                                self.push("hub.commit()")
+                        self.commit_changes()
                         raise EOFError
 
             shell = CustomShell(locals=locals)
@@ -257,24 +260,23 @@ class ToolboxCommand(CommandWithDB):
     desc = "Launch the TurboGears Toolbox"
 
     def __init__(self, version):
-        self.hostlist = ['127.0.0.1', '::1', '::ffff:127.0.0.1']
+        self.hostlist = ['127.0.0.1', '::1']
 
         parser = optparse.OptionParser(
             usage="%prog toolbox [options]",
-            version="%prog " + version)
-        parser.add_option("-n", "--no-open",
+            version='%prog ' + version)
+        parser.add_option('-n', '--no-open',
             help="don't open browser automatically",
-            dest="noopen", action="store_true", default=False)
-        parser.add_option("-c", "--add-client",
+            dest='noopen', action='store_true', default=False)
+        parser.add_option('-c', '--add-client',
             help="allow client ip address specified to connect to toolbox"
                 " (can be specified more than once)",
-            dest="host", action="append", default=None)
-        parser.add_option("-p", "--port",
+            dest='host', action='append', default=None)
+        parser.add_option('-p', '--port',
             help="port to run the Toolbox on",
-            dest="port", default=7654)
-        parser.add_option("--config",
-            help="config file", dest="config",
-            default=self.config or get_project_config())
+            dest='port', default=7654)
+        parser.add_option('--config', help="config file to use",
+            dest='config', default=self.config or get_project_config())
 
         options, args = parser.parse_args(sys.argv[1:])
         self.port = int(options.port)
@@ -284,16 +286,27 @@ class ToolboxCommand(CommandWithDB):
         if options.host:
             self.hostlist = self.hostlist + options.host
 
-        turbogears.widgets.load_widgets()
+        widgets.load_widgets()
 
     def openbrowser(self):
         import webbrowser
-        webbrowser.open("http://localhost:%d" % self.port)
+        webbrowser.open('http://localhost:%d' % self.port)
 
     def run(self):
         import cherrypy
         from turbogears import toolbox
 
+        # TODO: remove this check once we convert the whole toolbox to genshi
+        try:
+            import turbokid
+        except ImportError:
+            # we could not import turbokid, the toolbox will crash with
+            # horrible tracebacks...
+            print "Please easy_install turbokid, toolbox cannot run without it"
+            # sys exit with different than zero error code in case someone
+            # is using the error code to know if it worked...
+            sys.exit(2)
+
         # Make sure we have full configuration with every option
         # in it so other plugins or whatever find what they need
         # when starting even inside the toolblox
@@ -307,7 +320,7 @@ class ToolboxCommand(CommandWithDB):
         root = SecureObject(toolbox.Toolbox(), from_any_host(self.hostlist),
                 exclude=['noaccess'])
 
-        cherrypy.tree.mount(root, "/")
+        cherrypy.tree.mount(root, '/')
 
         # amend some parameters since we are running from the command
         # line in order to change port, log methods...
@@ -327,10 +340,10 @@ class ToolboxCommand(CommandWithDB):
             'kid.encoding': 'utf-8'
             }})
 
-        if not self.noopen:
-            cherrypy.server.start_with_callback(self.openbrowser)
-        else:
+        if self.noopen:
             cherrypy.server.start()
+        else:
+            cherrypy.server.start_with_callback(self.openbrowser)
 
 
 commands = None
@@ -339,9 +352,10 @@ def main():
     """Main command runner. Manages the primary command line arguments."""
     # add commands defined by entrypoints
     commands = {}
-    for entrypoint in pkg_resources.iter_entry_points("turbogears.command"):
+    for entrypoint in pkg_resources.iter_entry_points('turbogears.command'):
         command = entrypoint.load()
         commands[entrypoint.name] = (command.desc, entrypoint)
+    from turbogears import __version__
 
     def _help():
         """Custom help text for tg-admin."""
@@ -355,10 +369,10 @@ Options:
     -c CONFIG --config=CONFIG    Config file to use
     -e EGG_SPEC --egg=EGG_SPEC   Run command on given Egg
 
-Commands:""" % (turbogears.__version__, sys.argv[0])
+Commands:""" % (__version__, sys.argv[0])
 
         longest = max([len(key) for key in commands.keys()])
-        format = "%" + str(longest) + "s  %s"
+        format = '%' + str(longest) + 's  %s'
         commandlist = commands.keys()
         commandlist.sort()
         for key in commandlist:
@@ -366,13 +380,13 @@ Commands:""" % (turbogears.__version__, sys.argv[0])
 
     parser = optparse.OptionParser()
     parser.allow_interspersed_args = False
-    parser.add_option("-c", "--config", dest="config")
-    parser.add_option("-e", "--egg", dest="egg")
+    parser.add_option('-c', '--config', dest='config')
+    parser.add_option('-e', '--egg', dest='egg')
     parser.print_help = _help
     options, args = parser.parse_args(sys.argv[1:])
 
     # if command is not found display help
-    if not args or not commands.has_key(args[0]):
+    if not args or args[0] not in commands:
         _help()
         sys.exit()
 
@@ -386,8 +400,8 @@ Commands:""" % (turbogears.__version__, sys.argv[0])
         egg = pkg_resources.get_distribution(options.egg)
         os.chdir(egg.location)
 
-    if hasattr(command,"need_project"):
-        if not turbogears.util.get_project_name():
+    if hasattr(command, 'need_project'):
+        if not get_project_name():
             print "This command needs to be run from inside a project directory"
             return
         elif not options.config and not os.path.isfile(get_project_config()):
@@ -396,7 +410,7 @@ If it has been renamed use:
 tg-admin --config=<FILE> %s""" % commandname
             return
     command.config = options.config
-    command = command(turbogears.__version__)
+    command = command(__version__)
     command.run()
 
 
