@@ -9,17 +9,20 @@ from datetime import datetime
 import warnings
 
 import simplejson
+
+from formencode import ForEach
+from formencode import validators # to disambiguate the Number validator...
+from formencode import national
+
 from formencode.validators import *
 from formencode.compound import *
 from formencode.api import Invalid, NoDefault
 from formencode.schema import Schema
-from formencode import ForEach
 
-from turbogears.i18n import format
-from turbogears import util
 from turbojson import jsonify
 
-from formencode import validators # to disambiguate the Number validator...
+from turbogears import util
+from turbogears.i18n import format
 
 
 def _(s): return s # dummy
@@ -28,6 +31,29 @@ def _(s): return s # dummy
 Validator.gettextargs['domain'] = 'FormEncode'
 
 
+# Validators handling country names and/or languages
+
+def StateProvince(*kw, **kwargs):
+    warnings.warn("Please use national.USStateProvince",
+        DeprecationWarning, 2)
+    return national.USStateProvince(*kw, **kwargs)
+
+def PhoneNumber(*kw, **kwargs):
+    warnings.warn("Please use national.USPhoneNumber",
+        DeprecationWarning, 2)
+    return national.USPhoneNumber(*kw, **kwargs)
+
+def PostalCode(*kw, **kwargs):
+    warnings.warn("Please use national.USPostalCode",
+        DeprecationWarning, 2)
+    return national.USPostalCode(*kw, **kwargs)
+
+def IPhoneNumberValidator(*kw, **kwargs):
+    warnings.warn("Please use national.InternationalPhoneNumber",
+        DeprecationWarning, 2)
+    return national.InternationalPhoneNumber(*kw, **kwargs)
+
+
 class TgFancyValidator(FancyValidator):
     gettextargs = {'domain': 'TurboGears'}
 
@@ -40,14 +66,6 @@ class Money(TgFancyValidator):
         'empty': _('Empty values not allowed'),
     }
 
-    def __init__(self, allow_empty=None, *args, **kw):
-        if allow_empty is not None:
-            warnings.warn("Use not_empty instead of allow_empty",
-                          DeprecationWarning, 2)
-            not_empty = not allow_empty
-            kw["not_empty"] = not_empty
-        super(Money, self).__init__(*args, **kw)
-
     def _to_python(self, value, state):
         """Parse a string and return a float or integer."""
         try:
@@ -63,6 +81,8 @@ class Money(TgFancyValidator):
 class Number(TgFancyValidator):
     """Validate a decimal number."""
 
+    decimals = None
+
     def _to_python(self, value, state):
         """Parse a string and return a float or integer."""
         if isinstance(value, basestring):
@@ -73,10 +93,19 @@ class Number(TgFancyValidator):
         return validators.Number.to_python(value, state)
 
     def _from_python(self, value, state):
-        """Return a string using the correct grouping."""
-        dec_places = util.find_precision(value)
-        if dec_places > 0:
-            return format.format_decimal(value, dec_places)
+        """Return a string using the correct grouping.
+
+        If the validator was instantiated with a decimals parameter,
+        this is used for the number  of decimals. If None ist given,
+        the number of decimals is determined by the value itself.
+
+        """
+        if self.decimals is None:
+            decimals = util.find_precision(value)
+        else:
+            decimals = self.decimals
+        if decimals > 0:
+            return format.format_decimal(value, decimals)
         else:
             return format.format_number(value)
 
@@ -84,21 +113,13 @@ class Number(TgFancyValidator):
 class DateTimeConverter(TgFancyValidator):
     """Convert between Python datetime objects and strings."""
 
+    format="%Y/%m/%d %H:%M"
+
     messages = {
         'badFormat': _('Invalid datetime format'),
         'empty': _('Empty values not allowed'),
     }
 
-    def __init__(self, format="%Y/%m/%d %H:%M", allow_empty=None,
-                *args, **kwargs):
-        if allow_empty is not None:
-            warnings.warn("Use not_empty instead of allow_empty",
-                          DeprecationWarning, 2)
-            not_empty = not allow_empty
-            kwargs["not_empty"] = not_empty
-        super(DateTimeConverter, self).__init__(*args, **kwargs)
-        self.format = format
-
     def _to_python(self, value, state):
         """Parse a string and return a datetime object."""
         if value and isinstance(value, datetime):
