@@ -12,13 +12,16 @@ import tempfile
 
 from pkg_resources import resource_filename
 import formencode
-import kid
+# XXX: we need to make kid-support pluggable or such.
+try:
+    import kid
+except ImportError:
+    kid = None
 
 import turbogears
 import turbogears.i18n
 from turbogears import config
-from turbogears.toolbox.admi18n import pygettext, msgfmt, catalog
-from turbogears.toolbox.admi18n.catalog import normalize
+from turbogears.i18n.pygettext import pygettext, msgfmt, catalog
 from turbogears.command.base import silent_os_remove
 from turbogears.util import load_project_config, get_package_name
 
@@ -30,8 +33,13 @@ class ProgramError(StandardError):
 def copy_file(src, dest):
     if os.path.exists(dest):
         os.remove(dest)
-    data = open(src, 'rb').read()
-    open(dest, 'wb').write(data)
+    fh = open(src, 'rb')
+    data = fh.read()
+    fh.close()
+    fh = open(dest, 'wb')
+    fh.write(data)
+    fh.close()
+
 
 _str_literal = r"""(?:'((?:[^']|\\')*)'|"((?:[^"]|\\")*)")"""
 _py_i18n_re = re.compile(r"\b_\s*\(\s*[uU]?[rR]?%s\s*\)" % _str_literal)
@@ -72,10 +80,13 @@ Available commands:
             help="Force potentially damaging actions")
         parser.add_option("-a", "--ascii", default=False,
             action="store_true", dest="ascii_output",
-            help="Escape non-ascii characters")
+            help="Escape non-ascii characters (.py files only)")
         parser.add_option("-K", "--no-kid-support", default=True,
             action="store_false", dest="kid_support",
             help="Do not extract messages from Kid templates")
+        parser.add_option("-G", "--no-genshi-support", default=True,
+            action="store_false", dest="genshi_support",
+            help="Do not extract messages from Genshi templates")
         parser.add_option("", "--loose-kid-support",
             action="store_true", dest="loose_kid_support",
             help="Extract ALL messages from Kid templates (this is default)")
@@ -94,6 +105,9 @@ Available commands:
             default="static/javascript",
             help="Base directory of javascript files"
                 " for generated message-files.")
+        parser.add_option("-e", "--js-encoding",
+            dest="js_encoding", default="utf-8", metavar="ENCODING",
+            help="Encoding of JavaScript source files (default: UTF-8)")
         parser.set_defaults(loose_kid_support=True, js_support=True)
         self.parser = parser
 
@@ -117,7 +131,7 @@ Available commands:
 
         if os.path.exists(self.locale_dir) and \
                 not os.path.isdir(self.locale_dir):
-            raise ProgramError, (
+            raise ProgramError(
                 '%s is not a directory' % self.locale_dir)
 
         if not os.path.exists(self.locale_dir):
@@ -273,6 +287,7 @@ for(var i in _messages) {
     def scan_source_files(self):
         source_files = []
         kid_files = []
+        genshi_files = []
         js_files = []
         srcdir = self.options.source_dir or get_package_name().split('.', 1)[0]
         print 'Scanning source directory', srcdir
@@ -286,6 +301,8 @@ for(var i in _messages) {
                     source_files.append(srcfile)
                 elif ext == '.kid':
                     kid_files.append(srcfile)
+                elif ext == '.html':
+                    genshi_files.append(srcfile)
                 elif ext == '.js':
                     js_files.append(srcfile)
                 else:
@@ -301,13 +318,24 @@ for(var i in _messages) {
         pygettext.sys.argv = [''] + pygettext_options + source_files
         pygettext.main()
         if not os.path.exists(tmp_potfile):
-            raise ProgramError, 'pygettext failed'
+            raise ProgramError('pygettext failed')
         atexit.register(silent_os_remove, tmp_potfile)
+
         if kid_files and self.options.kid_support:
-            self.scan_kid_files(tmp_potfile, kid_files)
+            if not kid:
+                print "Kid not installed, no support for Kid templates."
+            else:
+                self.scan_kid_files(tmp_potfile, kid_files)
+
+        if genshi_files and self.options.genshi_support:
+            try:
+                self.scan_genshi_files(tmp_potfile, genshi_files)
+            except ImportError:
+                print "Genshi not installed, no support for Genshi templates."
 
         if js_files and self.options.js_support:
             self.scan_js_files(tmp_potfile, js_files)
+
         potfile = self.get_potfile_path()
         if os.path.isfile(potfile):
             bakfile = potfile.replace('.pot', '.bak')
@@ -317,7 +345,55 @@ for(var i in _messages) {
         os.rename(tmp_potfile, potfile)
         print 'Message templates written to', potfile
 
+    def scan_genshi_files(self, potfile, files):
+        """Extract strings from Genhsi templates and write to pot file.
+
+        Configuration settings:
+
+        `genshi.i18n.ignore_tags` -- `['script', 'style']`
+            List of element names. Content inside elements named in this list
+            is not extracted as translatable text. Can be a space-separated
+            string or a list of string.
+        `genshi.i18n.include_attrs` -- `['abbr', 'alt', 'label', 'prompt',
+                'standby', 'summary', 'title']`
+            List of attribute names. Only values of the attributes named in
+            this list are extracted as translatable text. Can be a
+            space-separated string or a list os strings.
+
+        See http://genshi.edgewall.org/wiki/Documentation/0.5.x/i18n.html for
+        more information.
+
+        """
+
+        extract_strings = turbogears.i18n.pygettext.pygettext.extract_genshi_strings
+        messages = []
+        options = {}
+        for opt in ('include_attrs', 'ignore_tags'):
+            val = config.get('genshi.i18n.' + opt)
+            if val:
+                options[opt] = val
+
+        for fname in files:
+            print 'Working on', fname
+            for msg in extract_strings(fname, options or None):
+                # Genshi 0.6 adds comments as additional 4th element of msg,
+                # so we take care to stay compatible
+                lineno, text = msg[0], msg[2]
+                if text:
+                    if isinstance(text, tuple): # e.g. for ngettext
+                        for subtext in text:
+                            if subtext:
+                                messages.append((lineno, fname, subtext))
+                    else:
+                        messages.append((lineno, fname, text))
+
+        self._write_potfile_entries(potfile, messages)
+
     def scan_kid_files(self, potfile, files):
+        """Extract strings from Genhsi templates and write to pot file."""
+
+        # XXX: This duplicates functionality already present in
+        #   turbogears.i18n.pygettext! This should be recfactored.
         messages = []
         tags_to_ignore = ['script', 'style']
         keys = []
@@ -336,7 +412,7 @@ for(var i in _messages) {
                     print "Warning: Mixed content in tag <%s>: %s" % (tag, k)
                 elif isinstance(parts, basestring):
                     key = k.strip()
-            if key and (key not in keys) and (tag not in tags_to_ignore):
+            if key and key not in keys and tag not in tags_to_ignore:
                 messages.append((tag, fname, key))
                 keys.append(key)
 
@@ -378,27 +454,22 @@ for(var i in _messages) {
                     if sentinel is None and el.strip():
                         process_text(False, el, tag)
             fh.close()
-        fd = open(potfile, 'at+')
-        for tag, fname, text in messages:
-            text = normalize(text.encode('utf-8'))
-            print >> fd, '#: %s:%s' % (fname, tag)
-            print >> fd, 'msgid %s' % text
-            print >> fd, 'msgstr ""'
-            print >> fd, ''
-        fd.close()
+        self._write_potfile_entries(potfile, messages)
 
     def get_strings_in_js(self, fname):
         messages = []
         keys = []
-        for i, line in enumerate(open(fname)):
-            s = _js_i18n_re.search(line)
+        fh = open(fname)
+        for i, line in enumerate(fh):
+            s = _js_i18n_re.search(line.decode(self.options.js_encoding))
             while s:
                 key = s.group(1) or s.group(2)
                 pos = s.end()
-                if key and (key not in keys):
+                if key and key not in keys:
                     messages.append((i + 1, fname, key))
                     keys.append(key)
                 s = _js_i18n_re.search(line, pos)
+        fh.close()
         return keys, messages
 
     def scan_js_files(self, potfile, files):
@@ -409,14 +480,18 @@ for(var i in _messages) {
             k, m = self.get_strings_in_js(fname)
             keys.extend(k)
             messages.extend(m)
-        fd = open(potfile, 'at+')
-        for linenumber, fname, text in messages:
-            text = normalize(text.encode('utf-8'))
-            print >> fd, '#: %s:%i' % (fname, linenumber)
-            print >> fd, 'msgid %s' % text
-            print >> fd, 'msgstr ""'
-            print >> fd, ''
-        fd.close()
+        self._write_potfile_entries(potfile, messages)
+
+    def _write_potfile_entries(self, potfile, messages):
+        if messages:
+            fd = open(potfile, 'at+')
+            for linenumber, fname, text in messages:
+                if text:
+                    text = catalog.normalize(text.encode('utf-8'))
+                    fd.write('#: %s:%s\n' % (fname, linenumber))
+                    fd.write('msgid %s\n' % text)
+                    fd.write('msgstr ""\n\n')
+            fd.close()
 
     def get_potfile_path(self):
         return os.path.join(self.locale_dir, '%s.pot' % self.domain)
@@ -443,7 +518,9 @@ for(var i in _messages) {
         This leads to later error when a program trying to read catalog.
 
         """
-        data = file(potfile, 'rb').read()
+        fh = open(potfile, 'rb')
+        data = fh.read()
+        fh.close()
         def repl(m):
             """Remove tzinfo if it breaks encoding."""
             tzinfo = m.group(2)
@@ -454,7 +531,9 @@ for(var i in _messages) {
             return m.group(0) # leave unchanged
         data = re.sub(
             "(POT-Creation-Date: [\d-]+ [0-9:]+)\+([^\\\\]+)", repl, data)
-        file(potfile, 'wb').write(data)
+        fh = open(potfile, 'wb')
+        fh.write(data)
+        fh.close()
 
 
 def main():
