@@ -4,11 +4,6 @@ from math import ceil
 import logging
 import warnings
 
-try:
-    set
-except NameError: # Python 2.3
-    from sets import Set as set
-
 import cherrypy
 try:
     import sqlobject
@@ -26,9 +21,11 @@ from turbogears.decorator import weak_signature_decorator
 from turbogears.view import variable_providers
 from formencode.variabledecode import variable_encode
 from turbogears.widgets import PaginateDataGrid
+from turbogears.util import add_tg_args
 
 log = logging.getLogger("turbogears.paginate")
 
+
 # lists of databases that lack support for OFFSET
 # this will need to be updated periodically as modules change
 _so_no_offset = 'mssql maxdb sybase'.split()
@@ -55,8 +52,8 @@ class itemwrapper:
         return obj[self.name]
 
 
-def paginate(var_name, default_order='', default_reversed=None, limit=10,
-            allow_limit_override=False, max_pages=5, dynamic_limit=None):
+def paginate(var_name, default_order='', limit=10,
+            max_limit=0, max_pages=5, max_sort=1000, dynamic_limit=None):
     """The famous TurboGears paginate decorator.
 
     @param var_name: the variable name that the paginate decorator will try
@@ -70,19 +67,16 @@ def paginate(var_name, default_order='', default_reversed=None, limit=10,
     @type default_order: string or a list of strings. Any string starting with
     "-" (minus sign) indicates a reverse order for that field/column.
 
-    @param default_reversed: Deprecated, use default_order with minus sign.
-    @type default_reversed: Boolean
-
-    @param limit: the hard coded limit that the paginate decorator will impose
+    @param limit: The hard-coded limit that the paginate decorator will impose
     on the number of "var_name" to display at the same time. This value can be
     overridden by the use of the dynamic_limit keyword argument.
     @type limit: integer
 
-    @param allow_limit_override: A boolean that indicates if the parameters
-    passed in the calling URL can modify the imposed limit. By default it is
-    set to False. If you want to be able to control the limit by using a URL
-    parameter then you need to set this to True.
-    @type allow_limit_override: Boolean
+    @param max_limit: The maximum number to which the imposed limit can be
+    increased using the "var_name"_tgp_limit keyword argument in the URL.
+    If this is set to 0, no dynamic change at all will be allowed;
+    if it is set to None, any change will be allowed.
+    @type max_limit: int
 
     @param max_pages: Used to generate the tg.paginate.pages variable. If the
     page count is larger than max_pages, tg.paginate.pages will only contain
@@ -90,8 +84,13 @@ def paginate(var_name, default_order='', default_reversed=None, limit=10,
     A zero value means that all pages will be shown, no matter how much.
     @type max_pages: integer
 
+    @param max_sort: The maximum number of records that will be sorted in
+    memory if the data cannot be sorted using SQL. If set to 0, sorting in
+    memory will never be performed; if set to None, no limit will be imposed.
+    @type max_sort: integer
+
     @param dynamic_limit: If specified, this parameter must be the name
-    of a key present in the dictionnary returned by your decorated
+    of a key present in the dictionary returned by your decorated
     controller. The value found for this key will be used as the limit
     for our pagination and will override the other settings, the hard-coded
     one declared in the decorator itself AND the URL parameter one.
@@ -103,20 +102,12 @@ def paginate(var_name, default_order='', default_reversed=None, limit=10,
 
     def entangle(func):
 
+        get = turbogears.config.get
+
         def decorated(func, *args, **kw):
 
-            get = turbogears.config.get
-
             def kwpop(name, default=None):
-                return kw.pop(var_name + '_tgp_' + name,
-                    kw.pop('tg_paginate_' + name, default))
-
-            if default_reversed is not None:
-                warnings.warn("default_reversed is deprecated."
-                    " Use default_order='-field' to indicate"
-                    " default reversed order, or"
-                    " default_order=['field1', '-field2, 'field3']"
-                    " for multiple fields.", DeprecationWarning, 2)
+                return kw.pop(var_name + '_tgp_' + name, default)
 
             page = kwpop('no')
             if page is None:
@@ -132,18 +123,20 @@ def paginate(var_name, default_order='', default_reversed=None, limit=10,
                     page = 1
                     if get('paginate.redirect_on_out_of_range'):
                         cherrypy.request.params[var_name + '_tgp_no'] = page
-                        redirect(cherrypy.request.path, cherrypy.request.params)
+                        redirect(cherrypy.request.path_info, cherrypy.request.params)
 
             try:
                 limit_ = int(kwpop('limit'))
-                if not allow_limit_override:
-                    raise ValueError
+                if max_limit is not None:
+                    if max_limit <= 0:
+                        raise ValueError
+                    limit_ = min(limit_, max_limit)
             except (TypeError, ValueError):
                 limit_ = limit
             order = kwpop('order')
             ordering = kwpop('ordering')
 
-            log.debug("pagiante params: page=%s, limit=%s, order=%s",
+            log.debug("paginate params: page=%s, limit=%s, order=%s",
                 page, limit_, order)
 
             # get the output from the decorated function
@@ -175,13 +168,7 @@ def paginate(var_name, default_order='', default_reversed=None, limit=10,
             else:
                 ordering = default_order or []
                 if isinstance(ordering, basestring):
-                    # adapt old style default_order to new style
-                    if default_reversed:
-                        ordering = "-" + ordering
                     ordering = [ordering]
-                elif default_reversed:
-                    raise ValueError("paginate: default_reversed (deprecated)"
-                        " only allowed when default_order is a basestring")
 
             if order:
                 order = str(order)
@@ -200,7 +187,8 @@ def paginate(var_name, default_order='', default_reversed=None, limit=10,
                     row_count = len(var_data)
 
             if ordering:
-                var_data = sort_data(var_data, ordering)
+                var_data = sort_data(var_data, ordering,
+                    max_sort is None or 0 < row_count <= max_sort)
 
             # If limit is zero then return all our rows
             if not limit_:
@@ -212,12 +200,12 @@ def paginate(var_name, default_order='', default_reversed=None, limit=10,
                 page = max(page_count, 1)
                 if get('paginate.redirect_on_last_page'):
                     cherrypy.request.params[var_name + '_tgp_no'] = page
-                    redirect(cherrypy.request.path, cherrypy.request.params)
+                    redirect(cherrypy.request.path_info, cherrypy.request.params)
             elif page > page_count:
                 page = max(page_count, 1)
                 if get('paginate.redirect_on_out_of_range'):
                     cherrypy.request.params[var_name + '_tgp_no'] = page
-                    redirect(cherrypy.request.path, cherrypy.request.params)
+                    redirect(cherrypy.request.path_info, cherrypy.request.params)
 
             offset = (page-1) * limit_
 
@@ -227,8 +215,7 @@ def paginate(var_name, default_order='', default_reversed=None, limit=10,
             input_values =  variable_encode(cherrypy.request.params.copy())
             input_values.pop('self', None)
             for input_key in input_values.keys():
-                if (input_key.startswith(var_name + '_tgp_') or
-                        input_key.startswith('tg_paginate_')):
+                if input_key.startswith(var_name + '_tgp_'):
                     del input_values[input_key]
 
             paginate_instance = Paginate(
@@ -249,7 +236,8 @@ def paginate(var_name, default_order='', default_reversed=None, limit=10,
 
             # we replace the var with the sliced one
             endpoint = offset + limit_
-            log.debug("slicing data between %d and %d", offset, endpoint)
+            log.debug("paginate: slicing data between %d and %d",
+                offset, endpoint)
 
             global _simulate_offset
             if _simulate_offset is None:
@@ -281,7 +269,13 @@ def paginate(var_name, default_order='', default_reversed=None, limit=10,
                         for i in xrange(offset, endpoint)]
 
             return output
+
+        if not get('tg.strict_parameters', False):
+            # add hint that paginate parameters shall be left intact
+            add_tg_args(func, (var_name + '_tgp_' + arg
+                for arg in ('no', 'limit', 'order', 'ordering')))
         return decorated
+
     return weak_signature_decorator(entangle)
 
 
@@ -328,15 +322,13 @@ class Paginate:
                 var_name + '_tgp_limit': limit
             })
             self.href_next = turbogears.url(
-                cherrypy.request.path,
-                self.input_values)
+                cherrypy.request.path_info, self.input_values)
             self.input_values.update({
                 var_name + '_tgp_no': 'last',
                 var_name + '_tgp_limit': limit
             })
             self.href_last = turbogears.url(
-                cherrypy.request.path,
-                self.input_values)
+                cherrypy.request.path_info, self.input_values)
         else:
             self.href_next = None
             self.href_last = None
@@ -347,15 +339,13 @@ class Paginate:
                 var_name + '_tgp_limit': limit
             })
             self.href_prev = turbogears.url(
-                cherrypy.request.path,
-                self.input_values)
+                cherrypy.request.path_info, self.input_values)
             self.input_values.update({
                 var_name + '_tgp_no': 1,
                 var_name + '_tgp_limit': limit
             })
             self.href_first = turbogears.url(
-                cherrypy.request.path,
-                self.input_values)
+                cherrypy.request.path_info, self.input_values)
         else:
             self.href_prev = None
             self.href_first = None
@@ -464,19 +454,23 @@ def sql_order_col(col, ascending=True):
         return sqlobject_order_col(col, ascending)
     raise TypeError("Expected Column, but got %s" % type(col))
 
-def sort_data(data, ordering):
+def sort_data(data, ordering, in_memory=True):
     """Sort data based on ordering.
 
     Tries to sort the data using SQL whenever possible,
-    otherwise sorts the data as list in memory.
+    otherwise sorts the data as list in memory unless in_memory is false.
 
     """
     try:
-        order_by = data.order_by # SQLAlchemy
+        order_by = data.order_by # SQLAlchemy, gets variable argument list
+        if hasattr(data, 'get_by'): # SQLAlchemy < 0.5
+            old_order_by = order_by # old version, gets single argument
+            order_by = lambda *cols: old_order_by(cols)
         get_column, order_col = sqlalchemy_get_column, sqlalchemy_order_col
     except AttributeError:
         try:
-            order_by = data.orderBy # SQLObject
+            orderBy = data.orderBy  # SQLObject, gets single argument
+            order_by = lambda *cols: orderBy(cols)
             get_column, order_col = sqlobject_get_column, sqlobject_order_col
         except AttributeError:
             order_by = None
@@ -491,7 +485,7 @@ def sort_data(data, ordering):
             descending = False
         if order_by:
             col = get_column(order, data)
-            if col:
+            if col is not None:
                 order_cols.append(order_col(col, descending))
                 continue
         if not order_cols:
@@ -501,44 +495,40 @@ def sort_data(data, ordering):
             else:
                 num_ascending += 1
     if order_by and order_cols:
-        data = order_by(order_cols)
+        data = order_by(*order_cols)
     if key_cols:
-        data = list(data)
-        if not data:
-            return data
-        wrapper = isinstance(data[0], dict) and itemwrapper or attrwrapper
-        keys = [(wrapper(col[0]), col[1]) for col in key_cols]
-        if num_ascending == 0 or num_descending == 0:
-            reverse = num_ascending == 0
-            keys = [key[0] for key in keys]
-            if len(key_cols) == 1:
-                key = keys[0]
-            else:
-                key = lambda row: [key(row) for key in keys]
-        else:
-            reverse = num_descending > num_ascending
-            def reverse_key(key, descending):
-                if reverse == descending:
-                    return key
+        if in_memory:
+            data = list(data)
+            if not data:
+                return data
+            wrapper = isinstance(data[0], dict) and itemwrapper or attrwrapper
+            keys = [(wrapper(col[0]), col[1]) for col in key_cols]
+            if num_ascending == 0 or num_descending == 0:
+                reverse = num_ascending == 0
+                keys = [key[0] for key in keys]
+                if len(key_cols) == 1:
+                    key = keys[0]
                 else:
-                    keys = map(key, data)
-                    try:
-                        keys = list(set(keys))
-                    except TypeError: # unhashable
-                        keys.sort()
-                        return lambda row: -keys.index(key(row))
-                    else:
-                        keys.sort()
-                        keys = dict([(k, -n) for n, k in enumerate(keys)])
-                        return lambda row: keys[key(row)]
-            keys = [reverse_key(*key) for key in keys]
-            key = lambda row: [key(row) for key in keys]
-        try:
-            data.sort(key=key, reverse=reverse)
-        except TypeError: # Python 2.3
-            if reverse:
-                cmpkey = lambda row1, row2: cmp(key(row2), key(row1))
+                    key = lambda row: [key(row) for key in keys]
             else:
-                cmpkey = lambda row1, row2: cmp(key(row1), key(row2))
-            data.sort(cmpkey)
+                reverse = num_descending > num_ascending
+                def reverse_key(key, descending):
+                    if reverse == descending:
+                        return key
+                    else:
+                        keys = map(key, data)
+                        try:
+                            keys = list(set(keys))
+                        except TypeError: # unhashable
+                            keys.sort()
+                            return lambda row: -keys.index(key(row))
+                        else:
+                            keys.sort()
+                            keys = dict((k, -n) for n, k in enumerate(keys))
+                            return lambda row: keys[key(row)]
+                keys = [reverse_key(*key) for key in keys]
+                key = lambda row: [key(row) for key in keys]
+            data.sort(key=key, reverse=reverse)
+        else:
+            log.debug("paginate: sorting in memory not allowed")
     return data
