@@ -1,37 +1,23 @@
 """Tests for paginate"""
 
-import unittest
 from urllib import quote
-import warnings
-
-from turbogears import config, expose, database
-from turbogears.controllers import RootController, url
-from turbogears.database import get_engine, metadata, session, mapper
-from turbogears.paginate import paginate, sort_ordering, sort_data
-from turbogears.testutil import create_request, sqlalchemy_cleanup
-from turbojson.jsonify import jsonify
 
 import cherrypy
+
 from sqlalchemy import Table, Column, ForeignKey, Integer, String
-from sqlalchemy.orm import relation
-try:
-    from sqlalchemy.ext.selectresults import SelectResults as SASelectResults
-except ImportError: # SQLAlchemy >= 0.5
-    SASelectResults = None
-from sqlobject import SQLObject, IntCol, StringCol, connectionForURI
+from sqlalchemy.orm import mapper, relation
+from sqlobject import SQLObject, IntCol, StringCol
 
-query_methods = 'Q QA SR SO SL'.split()
-if SASelectResults is None:
-    query_methods.remove('SR')
+from turbojson.jsonify import jsonify
 
-config.update({"sqlalchemy.dburi": "sqlite:///:memory:"})
+from turbogears import config, expose, database, testutil
+from turbogears.controllers import RootController
+from turbogears.database import bind_metadata, metadata, session
+from turbogears.paginate import paginate, sort_ordering, sort_data
+from turbogears.testutil import sqlalchemy_cleanup
 
-# sqlalchemy setup
-sqlalchemy_cleanup()
-get_engine()
+query_methods = 'Q QA SO SL'.split()
 
-# sqlobject setup
-database.set_db_uri("sqlite:///:memory:")
 __connection__ = hub = database.PackageHub('test_paginate')
 
 
@@ -185,6 +171,7 @@ class Spy(object):
         return "ok: [paginate %s ]" % \
             " , ".join(["%s=%r" % (k, v) for k, v in paginate.__dict__.items()])
 
+    @staticmethod
     def assert_ok(body, key, value, raw=False):
         assert "ok: [paginate" in body
         if raw:
@@ -193,175 +180,163 @@ class Spy(object):
             expr = "%s=%r " % (key, value)
         # some simplejson versions escape forward slashes, so fix this
         body = body.replace('\\/', '/')
-        if expr not in body:
-            print body
         assert expr in body, "expected %s" % expr
-    assert_ok = staticmethod(assert_ok)
 
-[jsonify.when('isinstance(obj, Spy)')]
+@jsonify.when('isinstance(obj, Spy)')
 def jsonify_spy(obj):
     result = str(obj)
     return result
 
 
-class TestSpy(unittest.TestCase):
+class TestPagination(testutil.TGTest):
+    """Base class for all Paginate TestCases"""
+
+    def setUp(self):
+        self.root = self.MyRoot
+        super(TestPagination, self).setUp()
+
+    def request(self, url, status=200):
+        response = self.app.get(url, status=status)
+        self.body = response.body
+        if "fail: " in self.body:
+            assert False, ("Spy alert! Check output:\n%s" % self.body)
+
+
+class TestSpy(TestPagination):
     """Never trust a spy"""
 
     class MyRoot(RootController):
 
-        [expose()]
-        [paginate("data")]
+        @expose()
+        @paginate("data")
         def spy(self):
             data = range(100)
             spy = Spy()
             return dict(data=data, spy=spy)
 
-        [expose()]
-        [paginate("data")]
+        @expose()
+        @paginate("data")
         def spy_correct_expectation(self):
             data = range(100)
             spy = Spy(page_count=10)
             return dict(data=data, spy=spy)
 
-        [expose()]
-        [paginate("data")]
+        @expose()
+        @paginate("data")
         def spy_wrong_expectation(self):
             data = range(100)
             spy = Spy(page_count=9)
             return dict(data=data, spy=spy)
 
-        [expose()]
-        [paginate("data")]
+        @expose()
+        @paginate("data")
         def spy_invalid_expectation(self):
             data = range(100)
             spy = Spy(foobar=10)
             return dict(data=data, spy=spy)
 
-
-    def setUp(self):
-        cherrypy.root = self.MyRoot()
-
     def test_spy(self):
-        create_request('/spy')
-        body = cherrypy.response.body[0]
-        Spy.assert_ok(body, 'current_page', 1)
+        response = self.app.get('/spy')
+        Spy.assert_ok(response.body, 'current_page', 1)
         try:
-            Spy.assert_ok(body, 'current_page', 2)
+            Spy.assert_ok(response.body, 'current_page', 2)
             raise Exception("above test should have failed")
         except AssertionError:
             pass
 
     def test_correct_expectation(self):
-        create_request('/spy_correct_expectation')
-        body = cherrypy.response.body[0]
-        assert "ok: [paginate" in body
+        response = self.app.get('/spy_correct_expectation')
+        assert "ok: [paginate" in response
 
     def test_wrong_expectation(self):
-        create_request('/spy_wrong_expectation')
-        body = cherrypy.response.body[0]
-        assert "fail: expected page_count=9, got page_count=10" in body
+        response = self.app.get('/spy_wrong_expectation')
+        assert "fail: expected page_count=9, got page_count=10" in response
 
     def test_invalid_expectation(self):
-        create_request('/spy_invalid_expectation')
-        body = cherrypy.response.body[0]
-        assert "fail: paginate does not have 'foobar' attribute" in body
+        response = self.app.get('/spy_invalid_expectation')
+        assert "fail: paginate does not have 'foobar' attribute" in response
 
     def test_raw_expectation(self):
-        create_request('/spy_correct_expectation')
-        Spy.assert_ok(cherrypy.response.body[0], 'var_name', 'data')
-        Spy.assert_ok(cherrypy.response.body[0], 'var_name', "'data'", raw=True)
-
-
-class TestPagination(unittest.TestCase):
-    """Base class for all Paginate TestCases"""
-
-    def setUp(self):
-        cherrypy.root = self.MyRoot()
-
-    def request(self, url):
-        create_request(url)
-        self.body = cherrypy.response.body[0]
-        self.status = cherrypy.response.status
-        if "fail: " in self.body:
-            print self.body
-            assert False, "Spy alert! Check body output for details..."
+        response = self.app.get('/spy_correct_expectation')
+        Spy.assert_ok(response.body, 'var_name', 'data')
+        Spy.assert_ok(response.body, 'var_name', "'data'", raw=True)
 
 
 class TestBasicPagination(TestPagination):
 
     class MyRoot(RootController):
 
-        [expose()]
-        [paginate("data", limit=4)]
+        @expose()
+        @paginate("data", limit=4)
         def basic(self):
             spy = Spy(var_name='data', pages=xrange(1, 4), limit=4, page_count=3,
                       order=None, ordering=[], row_count=10)
             data = range(10)
             return dict(data=data, spy=spy)
 
-        [expose()]
-        [paginate("data", limit=4)]
+        @expose()
+        @paginate("data", limit=4)
         def listlike(self):
             spy = Spy(var_name='data', pages=xrange(1, 4), limit=4, page_count=3,
                       order=None, ordering=[], row_count=10)
             data = listlike(range(10))
             return dict(data=data, spy=spy)
 
-        [expose()]
-        [paginate("data", limit=4, max_limit=None)]
+        @expose()
+        @paginate("data", limit=4, max_limit=None)
         def custom_limit(self):
             spy = Spy(var_name='data', order=None, ordering=[], row_count=10)
             data = range(10)
             return dict(data=data, spy=spy)
 
-        [expose()]
-        [paginate("data", limit=4, max_limit=7)]
+        @expose()
+        @paginate("data", limit=4, max_limit=7)
         def max_limit(self):
             spy = Spy(var_name='data', order=None, ordering=[], row_count=10)
             data = range(10)
             return dict(data=data, spy=spy)
 
-        [expose()]
-        [paginate("data")]
+        @expose()
+        @paginate("data")
         def default_max_pages(self):
             spy = Spy(var_name='data', limit=10, page_count=10,
                       order=None, ordering=[], row_count=100)
             data = range(100)
             return dict(data=data, spy=spy)
 
-        [expose()]
-        [paginate("data", max_pages=4)]
+        @expose()
+        @paginate("data", max_pages=4)
         def four_max_pages(self):
             spy = Spy(var_name='data', limit=10, page_count=10,
                       order=None, ordering=[], row_count=100)
             data = range(100)
             return dict(data=data, spy=spy)
 
-        [expose()]
-        [paginate("data", max_pages=3)]
+        @expose()
+        @paginate("data", max_pages=3)
         def three_max_pages(self):
             spy = Spy(var_name='data', limit=10, page_count=10,
                       order=None, ordering=[], row_count=100)
             data = range(100)
             return dict(data=data, spy=spy)
 
-        [expose()]
-        [paginate("data", limit=4, dynamic_limit='foobar')]
+        @expose()
+        @paginate("data", limit=4, dynamic_limit='foobar')
         def invalid_dynamic(self):
             data = range(10)
             return dict(data=data)
 
-        [expose()]
-        [paginate("data", limit=4, dynamic_limit='foobar', max_limit=None)]
+        @expose()
+        @paginate("data", limit=4, dynamic_limit='foobar', max_limit=None)
         def dynamic(self):
             spy = Spy(var_name='data', pages=xrange(1, 3), limit=7, page_count=2,
                       order=None, ordering=[], row_count=10)
             data = range(10)
             return dict(data=data, spy=spy, foobar=7)
 
-        [expose()]
-        [paginate("bar")]
-        [paginate("foo", limit=4)]
+        @expose()
+        @paginate("bar")
+        @paginate("foo", limit=4)
         def multiple(self):
             spy_foo = Spy(name='foo', var_name='foo', pages=xrange(1, 6),
                           limit=4, page_count=5, order=None, ordering=[],
@@ -372,8 +347,8 @@ class TestBasicPagination(TestPagination):
             bar = list('ABCDEFGHIJKLMNOPQRSTUVWXYZ') # 26 letters
             return dict(foo=foo, bar=bar, spy_foo=spy_foo, spy_bar=spy_bar)
 
-        [expose()]
-        [paginate("data", limit=4, max_limit=None)]
+        @expose()
+        @paginate("data", limit=4, max_limit=None)
         def empty(self):
             spy = Spy(var_name='data', pages=xrange(1, 1), page_count=0,
                       order=None, ordering=[], row_count=0,
@@ -382,32 +357,7 @@ class TestBasicPagination(TestPagination):
             return dict(data=data, spy=spy)
 
 
-    def test_pagination_old_style(self):
-        self.request("/basic")
-        assert '"data": [0, 1, 2, 3]' in self.body
-        Spy.assert_ok(self.body, 'current_page', 1)
-        Spy.assert_ok(self.body, 'first_item', 1)
-        Spy.assert_ok(self.body, 'last_item', 4)
-
-        self.request("/basic?tg_paginate_no=1")
-        assert '"data": [0, 1, 2, 3]' in self.body
-        Spy.assert_ok(self.body, 'current_page', 1)
-        Spy.assert_ok(self.body, 'first_item', 1)
-        Spy.assert_ok(self.body, 'last_item', 4)
-
-        self.request("/basic?tg_paginate_no=2")
-        assert '"data": [4, 5, 6, 7]' in self.body
-        Spy.assert_ok(self.body, 'current_page', 2)
-        Spy.assert_ok(self.body, 'first_item', 5)
-        Spy.assert_ok(self.body, 'last_item', 8)
-
-        self.request("/basic?tg_paginate_no=3")
-        assert '"data": [8, 9]' in self.body
-        Spy.assert_ok(self.body, 'current_page', 3)
-        Spy.assert_ok(self.body, 'first_item', 9)
-        Spy.assert_ok(self.body, 'last_item', 10)
-
-    def test_pagination_new_style(self):
+    def test_pagination(self):
         self.request("/basic?data_tgp_no=1")
         assert '"data": [0, 1, 2, 3]' in self.body
         Spy.assert_ok(self.body, 'current_page', 1)
@@ -434,11 +384,7 @@ class TestBasicPagination(TestPagination):
         Spy.assert_ok(self.body, 'last_item', 8)
 
     def test_limit_override(self):
-        # can't override limit (old style)
-        self.request('/basic?tg_paginate_limit=2')
-        assert '"data": [0, 1, 2, 3]' in self.body
-
-        # can't override limit (new style)
+        # can't override limit
         self.request("/basic?data_tgp_limit=2")
         assert '"data": [0, 1, 2, 3]' in self.body
 
@@ -449,14 +395,7 @@ class TestBasicPagination(TestPagination):
         Spy.assert_ok(self.body, 'limit', 7)
         Spy.assert_ok(self.body, 'pages', xrange(1, 3))
 
-        # can override limit (old style)
-        self.request('/custom_limit?tg_paginate_limit=2')
-        assert '"data": [0, 1]' in self.body
-        Spy.assert_ok(self.body, 'page_count', 5)
-        Spy.assert_ok(self.body, 'limit', 2)
-        Spy.assert_ok(self.body, 'pages', xrange(1, 6))
-
-        # can override limit (new style)
+        # can override limit
         self.request("/custom_limit?data_tgp_limit=2")
         assert '"data": [0, 1]' in self.body
         Spy.assert_ok(self.body, 'page_count', 5)
@@ -499,53 +438,35 @@ class TestBasicPagination(TestPagination):
     def test_strict_parameters(self):
         config.update({'tg.strict_parameters': True})
         try:
-            # old style
-            self.request("/basic?tg_paginate_no=2")
-            assert self.status.startswith('200')
+            # check that paginate works with strict parameters as well
+            self.request("/basic?data_tgp_no=2", status=200)
             assert '"data": [4, 5, 6, 7]' in self.body
             Spy.assert_ok(self.body, 'current_page', 2)
-            self.request('/custom_limit?tg_paginate_limit=2')
-            assert self.status.startswith('200')
+            self.request('/custom_limit?data_tgp_limit=2', status=200)
             assert '"data": [0, 1]' in self.body
             Spy.assert_ok(self.body, 'limit', 2)
-            self.request("/default_max_pages?tg_paginate_no=5")
-            assert self.status.startswith('200')
+            self.request("/default_max_pages?data_tgp_no=5", status=200)
             assert '"data": [40, 41, 42, ' in self.body
             Spy.assert_ok(self.body, 'pages', xrange(3, 8))
-            self.request("/basic?tg_paginate_rubbish=2")
-            assert self.status.startswith('500')
-            assert ("basic() got an unexpected keyword argument"
-                " 'tg_paginate_rubbish'") in self.body
-            # new style
-            self.request("/basic?data_tgp_no=2")
-            assert self.status.startswith('200')
-            assert '"data": [4, 5, 6, 7]' in self.body
-            Spy.assert_ok(self.body, 'current_page', 2)
-            self.request('/custom_limit?data_tgp_limit=2')
-            assert self.status.startswith('200')
-            assert '"data": [0, 1]' in self.body
-            Spy.assert_ok(self.body, 'limit', 2)
-            self.request("/default_max_pages?data_tgp_no=5")
-            assert self.status.startswith('200')
-            assert '"data": [40, 41, 42, ' in self.body
-            Spy.assert_ok(self.body, 'pages', xrange(3, 8))
-            self.request("/basic?data_tgp_rubbish=2")
-            assert self.status.startswith('500')
+            # check that wrong and old style parameters are not accepted
+            self.request("/basic?data_tgp_rubbish=2", status = 500)
             assert ("basic() got an unexpected keyword argument"
                 " 'data_tgp_rubbish'") in self.body
+            self.request("/basic?tg_paginate_no=2", status=500)
+            assert ("basic() got an unexpected keyword argument"
+                " 'tg_paginate_no'") in self.body
         finally:
             config.update({'tg.strict_parameters': False})
 
     def test_invalid_dynamic_limit(self):
-        self.request("/invalid_dynamic")
-        assert self.status.startswith("500")
+        self.request("/invalid_dynamic", status=500)
         assert 'paginate: dynamic_limit (foobar) not found in output dict' in self.body
 
     def test_dynamic_limit(self):
         self.request("/dynamic")
         assert '"data": [0, 1, 2, 3, 4, 5, 6]' in self.body
 
-        self.request('/dynamic?tg_paginate_limit=2')
+        self.request('/dynamic?data_tgp_limit=2')
         assert '"data": [0, 1, 2, 3, 4, 5, 6]' in self.body
 
     def test_multiple(self):
@@ -567,21 +488,21 @@ class TestBasicPagination(TestPagination):
 
     def test_out_of_bound_pages(self):
         for number in range(-3,1):
-            self.request("/basic?tg_paginate_no=%s" % number)
+            self.request("/basic?data_tgp_no=%s" % number)
             assert '"data": [0, 1, 2, 3]' in self.body
             Spy.assert_ok(self.body, 'current_page', 1)
             Spy.assert_ok(self.body, 'first_item', 1)
             Spy.assert_ok(self.body, 'last_item', 4)
 
         for number in range(4, 7):
-            self.request("/basic?tg_paginate_no=%s" % number)
+            self.request("/basic?data_tgp_no=%s" % number)
             assert '"data": [8, 9]' in self.body
             Spy.assert_ok(self.body, 'current_page', 3)
             Spy.assert_ok(self.body, 'first_item', 9)
             Spy.assert_ok(self.body, 'last_item', 10)
 
     def test_last_page(self):
-        self.request("/basic?tg_paginate_no=last")
+        self.request("/basic?data_tgp_no=last")
         assert '"data": [8, 9]' in self.body
         Spy.assert_ok(self.body, 'current_page', 3)
         Spy.assert_ok(self.body, 'first_item', 9)
@@ -634,7 +555,7 @@ class TestBasicPagination(TestPagination):
     def test_with_webpath(self):
         config.update({'server.webpath': '/web/path'})
         try:
-            self.request("/web/path/basic?data_tgp_no=2")
+            self.request("/basic?data_tgp_no=2")
             Spy.assert_ok(self.body, 'current_page', 2)
             Spy.assert_ok(self.body, 'href_first',
                 r"'/web/path/basic?data_tgp_no=1&data_tgp_limit=4'", raw=True)
@@ -754,7 +675,7 @@ def create_tables():
 
     # SQLObject tests will need only "addresses"
     for kw in addresses:
-        print SOAddress(**kw)
+        SOAddress(**kw)
 
 def drop_tables():
     metadata.drop_all()
@@ -769,8 +690,6 @@ class TestDatabasePagination(TestPagination):
                 data = session.query(Address)
             elif method == 'QA':
                 data = session.query(Address).all()
-            elif method == 'SR':
-                data = SASelectResults(session.query(Address))
             elif method == 'SO':
                 data = SOAddress.select()
             elif method == 'SL':
@@ -784,65 +703,56 @@ class TestDatabasePagination(TestPagination):
 
         # default_order = basestring
         basic1 = paginate("data", default_order="id")(__common)
-        basic1 = expose("turbogears.tests.paginate")(basic1)
+        basic1 = expose("kid:turbogears.tests.paginate")(basic1)
 
         # default_order = list
         basic2 = paginate("data", default_order=["id"])(__common)
-        basic2 = expose("turbogears.tests.paginate")(basic2)
-
-        # default_order = basestring with default_reversed (deprecated)
-        warnings.filterwarnings('ignore',
-            'default_reversed is deprecated', DeprecationWarning)
-        reversed1 = paginate("data", default_order="id", default_reversed=True)(__common)
-        reversed1 = expose("turbogears.tests.paginate")(reversed1)
+        basic2 = expose("kid:turbogears.tests.paginate")(basic2)
 
         # default_order = basestring
-        reversed2 = paginate("data", default_order="-id")(__common)
-        reversed2 = expose("turbogears.tests.paginate")(reversed2)
-
-        # default_order = list with default_reversed (WRONG!)
-        wrong_reversed = paginate("data", default_order=["id"], default_reversed=True)(__common)
-        wrong_reversed = expose("turbogears.tests.paginate")(wrong_reversed)
+        reversed1 = paginate("data", default_order="-id")(__common)
+        reversed1 = expose("kid:turbogears.tests.paginate")(reversed1)
 
         # default_order = list
-        reversed3 = paginate("data", default_order=["-id"])(__common)
-        reversed3 = expose("turbogears.tests.paginate")(reversed3)
+        reversed2 = paginate("data", default_order=["-id"])(__common)
+        reversed2 = expose("kid:turbogears.tests.paginate")(reversed2)
 
         # +/+
         default_compound_ordering1 = paginate("data", default_order=["city", "street"])(__common)
-        default_compound_ordering1 = expose("turbogears.tests.paginate")(default_compound_ordering1)
+        default_compound_ordering1 = expose("kid:turbogears.tests.paginate")(default_compound_ordering1)
 
         # +/-
         default_compound_ordering2 = paginate("data", default_order=["city", "-street"])(__common)
-        default_compound_ordering2 = expose("turbogears.tests.paginate")(default_compound_ordering2)
+        default_compound_ordering2 = expose("kid:turbogears.tests.paginate")(default_compound_ordering2)
 
         # -/+
         default_compound_ordering3 = paginate("data", default_order=["-city", "street"])(__common)
-        default_compound_ordering3 = expose("turbogears.tests.paginate")(default_compound_ordering3)
+        default_compound_ordering3 = expose("kid:turbogears.tests.paginate")(default_compound_ordering3)
 
         # -/-
         default_compound_ordering4 = paginate("data", default_order=["-city", "-street"])(__common)
-        default_compound_ordering4 = expose("turbogears.tests.paginate")(default_compound_ordering4)
+        default_compound_ordering4 = expose("kid:turbogears.tests.paginate")(default_compound_ordering4)
 
-        [expose("turbogears.tests.paginate")]
-        [paginate("data", default_order="id")]
+        @expose("kid:turbogears.tests.paginate")
+        @paginate("data", default_order="id")
         def related(self):
             # only SA Query
-            data = session.query(Address).outerjoin(['user', 'occupation'])
+            tables = ['user', 'occupation']
+            if hasattr(Address, 'c'): # SQLAlchemy < 0.5
+                tables = [tables]
+            data = session.query(Address).outerjoin(*tables)
 
             spy = Spy(var_name='data', pages=xrange(1, 3), limit=10, page_count=2,
                       order=None, row_count=16)
             return dict(data=data, spy=spy)
 
-        [expose("turbogears.tests.paginate")]
-        [paginate("data", default_order="id", limit=0)]
+        @expose("kid:turbogears.tests.paginate")
+        @paginate("data", default_order="id", limit=0)
         def zero_limit(self, method=None):
             if method == 'Q':
                 data = session.query(Address)
             elif method == 'QA':
                 data = session.query(Address).all()
-            elif method == 'SR':
-                data = SASelectResults(session.query(Address))
             elif method == 'SO':
                 data = SOAddress.select()
             elif method == 'SL':
@@ -854,8 +764,8 @@ class TestDatabasePagination(TestPagination):
                       order=None, row_count=16)
             return dict(data=data, spy=spy)
 
-        [expose("turbogears.tests.paginate")]
-        [paginate("data", default_order="id")]
+        @expose("kid:turbogears.tests.paginate")
+        @paginate("data", default_order="id")
         def empty_with_groupby(self):
             data = session.query(Address)
             try:
@@ -869,8 +779,6 @@ class TestDatabasePagination(TestPagination):
 
     def assert_order(self, *args):
         expr = 'data="%s"' % ''.join(['[Address %r]' % x for x in args])
-        if expr not in self.body:
-            print self.body
         assert expr in self.body, "expected %s" % expr
 
     def test_basic(self):
@@ -883,15 +791,6 @@ class TestDatabasePagination(TestPagination):
                 Spy.assert_ok(self.body, 'last_item', 10)
                 Spy.assert_ok(self.body, 'ordering', ['id'])
 
-                # old style
-                self.request("/%s?method=%s&tg_paginate_no=2" % (test, method))
-                self.assert_order(11, 12, 13, 14, 15, 16)
-                Spy.assert_ok(self.body, 'current_page', 2)
-                Spy.assert_ok(self.body, 'first_item', 11)
-                Spy.assert_ok(self.body, 'last_item', 16)
-                Spy.assert_ok(self.body, 'ordering', ['id'])
-
-                # new style
                 self.request("/%s?method=%s&data_tgp_no=2" % (test, method))
                 self.assert_order(11, 12, 13, 14, 15, 16)
                 Spy.assert_ok(self.body, 'current_page', 2)
@@ -902,7 +801,7 @@ class TestDatabasePagination(TestPagination):
     def test_ordering(self):
         for test in "basic1", "basic2":
             for method in query_methods:
-                self.request("/%s?method=%s&tg_paginate_ordering=%s" % \
+                self.request("/%s?method=%s&data_tgp_ordering=%s" % \
                     (test, method, 'street'))
                 self.assert_order(16, 2, 3, 4, 5, 6, 7, 8, 9, 10)
                 Spy.assert_ok(self.body, 'current_page', 1)
@@ -910,7 +809,7 @@ class TestDatabasePagination(TestPagination):
                 Spy.assert_ok(self.body, 'last_item', 10)
                 Spy.assert_ok(self.body, 'ordering', ['street'])
 
-                self.request("/%s?method=%s&tg_paginate_no=2&tg_paginate_ordering=%s" % \
+                self.request("/%s?method=%s&data_tgp_no=2&data_tgp_ordering=%s" % \
                     (test, method, 'street'))
                 self.assert_order(11, 12, 13, 14, 15, 1)
                 Spy.assert_ok(self.body, 'current_page', 2)
@@ -921,20 +820,19 @@ class TestDatabasePagination(TestPagination):
     def test_strict_parameters(self):
         config.update({'tg.strict_parameters': True})
         try:
-            # old style
-            self.request("/basic1?method=Q&tg_paginate_ordering=street")
-            self.assert_order(16, 2, 3, 4, 5, 6, 7, 8, 9, 10)
-            Spy.assert_ok(self.body, 'ordering', ['street'])
-            # new style
-            self.request("/basic1?method=Q&data_tgp_ordering=street")
-            assert self.status.startswith('200')
+            # check that paginate ordering works with strict parameters as well
+            self.request("/basic1?method=Q&data_tgp_ordering=street", status=200)
             self.assert_order(16, 2, 3, 4, 5, 6, 7, 8, 9, 10)
             Spy.assert_ok(self.body, 'ordering', ['street'])
+            # check that old style ordering is not accepted any more
+            self.request("/basic1?method=Q&tg_paginate_ordering=street", status=500)
+            assert ("got an unexpected keyword argument"
+                " 'tg_paginate_ordering'") in self.body
         finally:
             config.update({'tg.strict_parameters': False})
 
-    def test_default_reversed(self):
-        for test in "reversed1", "reversed2", "reversed3":
+    def test_reversed(self):
+        for test in "reversed1", "reversed2":
             for method in query_methods:
                 self.request("/%s?method=%s" % (test, method))
                 self.assert_order(16, 15, 14, 13, 12, 11, 10, 9, 8, 7)
@@ -950,22 +848,15 @@ class TestDatabasePagination(TestPagination):
                 Spy.assert_ok(self.body, 'last_item', 16)
                 Spy.assert_ok(self.body, 'ordering', ['-id'])
 
-    def test_invalid_default_reversed(self):
-        for method in query_methods:
-            self.request("/wrong_reversed?method=%s" % method)
-            assert self.status.startswith("500")
-            assert ('paginate: default_reversed (deprecated) only allowed'
-                ' when default_order is a basestring') in self.body
-
     def test_reverse_ordering(self):
         for test in "basic1", "basic2":
             for method in query_methods:
-                self.request("/%s?method=%s&tg_paginate_ordering=%s"
+                self.request("/%s?method=%s&data_tgp_ordering=%s"
                     % (test, method, '-street'))
                 self.assert_order(1, 15, 14, 13, 12, 11, 10, 9, 8, 7)
                 Spy.assert_ok(self.body, 'ordering', ['-street'])
 
-                self.request("/%s?method=%s&tg_paginate_no=2&tg_paginate_ordering=%s"
+                self.request("/%s?method=%s&data_tgp_no=2&data_tgp_ordering=%s"
                     % (test, method, '-street'))
                 self.assert_order(6, 5, 4, 3, 2, 16)
                 Spy.assert_ok(self.body, 'ordering', ['-street'])
@@ -974,45 +865,45 @@ class TestDatabasePagination(TestPagination):
         for test in "basic1", "basic2":
             for method in query_methods:
                 # +/+
-                self.request("/%s?method=%s&tg_paginate_ordering=%s"
+                self.request("/%s?method=%s&data_tgp_ordering=%s"
                     % (test, method, quote('city,street')))
                 self.assert_order(16, 3, 9, 10, 12, 13, 1, 4, 7, 8)
                 Spy.assert_ok(self.body, 'ordering', ['city', 'street'])
 
-                self.request("/%s?method=%s&tg_paginate_no=2&tg_paginate_ordering=%s"
+                self.request("/%s?method=%s&data_tgp_no=2&data_tgp_ordering=%s"
                     % (test, method, quote('city,street')))
                 self.assert_order(14, 15, 2, 5, 6, 11)
                 Spy.assert_ok(self.body, 'ordering', ['city', 'street'])
 
                 # +/-
-                self.request("/%s?method=%s&tg_paginate_ordering=%s"
+                self.request("/%s?method=%s&data_tgp_ordering=%s"
                     % (test, method, quote('city,-street')))
                 self.assert_order(1, 13, 12, 10, 9, 3, 16, 15, 14, 8)
                 Spy.assert_ok(self.body, 'ordering', ['city', '-street'])
 
-                self.request("/%s?method=%s&tg_paginate_no=2&tg_paginate_ordering=%s"
+                self.request("/%s?method=%s&data_tgp_no=2&data_tgp_ordering=%s"
                     % (test, method, quote('city,-street')))
                 self.assert_order(7, 4, 11, 6, 5, 2)
                 Spy.assert_ok(self.body, 'ordering', ['city', '-street'])
 
                 # -/+
-                self.request("/%s?method=%s&tg_paginate_ordering=%s"
+                self.request("/%s?method=%s&data_tgp_ordering=%s"
                     % (test, method, quote('-city,street')))
                 self.assert_order(2, 5, 6, 11, 4, 7, 8, 14, 15, 16)
                 Spy.assert_ok(self.body, 'ordering', ['-city', 'street'])
 
-                self.request("/%s?method=%s&tg_paginate_no=2&tg_paginate_ordering=%s"
+                self.request("/%s?method=%s&data_tgp_no=2&data_tgp_ordering=%s"
                     % (test, method, quote('-city,street')))
                 self.assert_order(3, 9, 10, 12, 13, 1)
                 Spy.assert_ok(self.body, 'ordering', ['-city', 'street'])
 
                 # -/-
-                self.request("/%s?method=%s&tg_paginate_ordering=%s"
+                self.request("/%s?method=%s&data_tgp_ordering=%s"
                     % (test, method, quote('-city,-street')))
                 self.assert_order(11, 6, 5, 2, 15, 14, 8, 7, 4, 1)
                 Spy.assert_ok(self.body, 'ordering', ['-city', '-street'])
 
-                self.request("/%s?method=%s&tg_paginate_no=2&tg_paginate_ordering=%s"
+                self.request("/%s?method=%s&data_tgp_no=2&data_tgp_ordering=%s"
                     % (test, method, quote('-city,-street')))
                 self.assert_order(13, 12, 10, 9, 3, 16)
                 Spy.assert_ok(self.body, 'ordering', ['-city', '-street'])
@@ -1024,7 +915,7 @@ class TestDatabasePagination(TestPagination):
             self.assert_order(16, 3, 9, 10, 12, 13, 1, 4, 7, 8)
             Spy.assert_ok(self.body, 'ordering', ['city', 'street'])
 
-            self.request("/default_compound_ordering1?method=%s&tg_paginate_no=2" % method)
+            self.request("/default_compound_ordering1?method=%s&data_tgp_no=2" % method)
             self.assert_order(14, 15, 2, 5, 6, 11)
             Spy.assert_ok(self.body, 'ordering', ['city', 'street'])
 
@@ -1035,7 +926,7 @@ class TestDatabasePagination(TestPagination):
             self.assert_order(1, 13, 12, 10, 9, 3, 16, 15, 14, 8)
             Spy.assert_ok(self.body, 'ordering', ['city', '-street'])
 
-            self.request("/default_compound_ordering2?method=%s&tg_paginate_no=2" % method)
+            self.request("/default_compound_ordering2?method=%s&data_tgp_no=2" % method)
             self.assert_order(7, 4, 11, 6, 5, 2)
             Spy.assert_ok(self.body, 'ordering', ['city', '-street'])
 
@@ -1046,7 +937,7 @@ class TestDatabasePagination(TestPagination):
             self.assert_order(2, 5, 6, 11, 4, 7, 8, 14, 15, 16)
             Spy.assert_ok(self.body, 'ordering', ['-city', 'street'])
 
-            self.request("/default_compound_ordering3?method=%s&tg_paginate_no=2" % method)
+            self.request("/default_compound_ordering3?method=%s&data_tgp_no=2" % method)
             self.assert_order(3, 9, 10, 12, 13, 1)
             Spy.assert_ok(self.body, 'ordering', ['-city', 'street'])
 
@@ -1057,92 +948,92 @@ class TestDatabasePagination(TestPagination):
             self.assert_order(11, 6, 5, 2, 15, 14, 8, 7, 4, 1)
             Spy.assert_ok(self.body, 'ordering', ['-city', '-street'])
 
-            self.request("/default_compound_ordering4?method=%s&tg_paginate_no=2" % method)
+            self.request("/default_compound_ordering4?method=%s&data_tgp_no=2" % method)
             self.assert_order(13, 12, 10, 9, 3, 16)
             Spy.assert_ok(self.body, 'ordering', ['-city', '-street'])
 
     def test_related_objects_ordering_level_1(self):
         # +/+
-        self.request("/related?tg_paginate_ordering=%s" % quote('user.name,id'))
+        self.request("/related?data_tgp_ordering=%s" % quote('user.name,id'))
         self.assert_order(1, 5, 3, 13, 7, 11, 16, 2, 12, 14)
         Spy.assert_ok(self.body, 'ordering', ['user.name', 'id'])
 
-        self.request("/related?tg_paginate_no=2&tg_paginate_ordering=%s"
+        self.request("/related?data_tgp_no=2&data_tgp_ordering=%s"
             % quote('user.name,id'))
         self.assert_order(10, 4, 6, 9, 15, 8)
         Spy.assert_ok(self.body, 'ordering', ['user.name', 'id'])
 
         # +/-
-        self.request("/related?tg_paginate_ordering=%s" % quote('user.name,-id'))
+        self.request("/related?data_tgp_ordering=%s" % quote('user.name,-id'))
         self.assert_order(5, 1, 3, 13, 16, 11, 7, 12, 2, 14)
         Spy.assert_ok(self.body, 'ordering', ['user.name', '-id'])
 
-        self.request("/related?tg_paginate_no=2&tg_paginate_ordering=%s"
+        self.request("/related?data_tgp_no=2&data_tgp_ordering=%s"
             % quote('user.name,-id'))
         self.assert_order(10, 15, 9, 6, 4, 8)
         Spy.assert_ok(self.body, 'ordering', ['user.name', '-id'])
 
         # -/+
-        self.request("/related?tg_paginate_ordering=%s" % quote('-user.name,id'))
+        self.request("/related?data_tgp_ordering=%s" % quote('-user.name,id'))
         self.assert_order(8, 4, 6, 9, 15, 10, 14, 2, 12, 7)
         Spy.assert_ok(self.body, 'ordering', ['-user.name', 'id'])
 
-        self.request("/related?tg_paginate_no=2&tg_paginate_ordering=%s"
+        self.request("/related?data_tgp_no=2&data_tgp_ordering=%s"
             % quote('-user.name,id'))
         self.assert_order(11, 16, 13, 3, 1, 5)
         Spy.assert_ok(self.body, 'ordering', ['-user.name', 'id'])
 
         # -/-
-        self.request("/related?tg_paginate_ordering=%s" % quote('-user.name,-id'))
+        self.request("/related?data_tgp_ordering=%s" % quote('-user.name,-id'))
         self.assert_order(8, 15, 9, 6, 4, 10, 14, 12, 2, 16)
         Spy.assert_ok(self.body, 'ordering', ['-user.name', '-id'])
 
-        self.request("/related?tg_paginate_no=2&tg_paginate_ordering=%s"
+        self.request("/related?data_tgp_no=2&data_tgp_ordering=%s"
             % quote('-user.name,-id'))
         self.assert_order(11, 7, 13, 3, 5, 1)
         Spy.assert_ok(self.body, 'ordering', ['-user.name', '-id'])
 
     def test_related_objects_ordering_level_2(self):
         # +/+
-        self.request("/related?tg_paginate_ordering=%s"
+        self.request("/related?data_tgp_ordering=%s"
             % quote('user.occupation.name,id'))
         self.assert_order(1, 5, 10, 13, 14, 2, 7, 11, 12, 16)
         Spy.assert_ok(self.body, 'ordering', ['user.occupation.name', 'id'])
 
-        self.request("/related?tg_paginate_no=2&tg_paginate_ordering=%s"
+        self.request("/related?data_tgp_no=2&data_tgp_ordering=%s"
             % quote('user.occupation.name,id'))
         self.assert_order(4, 6, 8, 9, 15, 3)
         Spy.assert_ok(self.body, 'ordering', ['user.occupation.name', 'id'])
 
         # +/-
-        self.request("/related?tg_paginate_ordering=%s"
+        self.request("/related?data_tgp_ordering=%s"
             % quote('user.occupation.name,-id'))
         self.assert_order(14, 13, 10, 5, 1, 16, 12, 11, 7, 2)
         Spy.assert_ok(self.body, 'ordering', ['user.occupation.name', '-id'])
 
-        self.request("/related?tg_paginate_no=2&tg_paginate_ordering=%s"
+        self.request("/related?data_tgp_no=2&data_tgp_ordering=%s"
             % quote('user.occupation.name,-id'))
         self.assert_order(15, 9, 8, 6, 4, 3)
         Spy.assert_ok(self.body, 'ordering', ['user.occupation.name', '-id'])
 
         # -/+
-        self.request("/related?tg_paginate_ordering=%s"
+        self.request("/related?data_tgp_ordering=%s"
             % quote('-user.occupation.name,id'))
         self.assert_order(3, 4, 6, 8, 9, 15, 2, 7, 11, 12)
         Spy.assert_ok(self.body, 'ordering', ['-user.occupation.name', 'id'])
 
-        self.request("/related?tg_paginate_no=2&tg_paginate_ordering=%s"
+        self.request("/related?data_tgp_no=2&data_tgp_ordering=%s"
             % quote('-user.occupation.name,id'))
         self.assert_order(16, 1, 5, 10, 13, 14)
         Spy.assert_ok(self.body, 'ordering', ['-user.occupation.name', 'id'])
 
         # -/-
-        self.request("/related?tg_paginate_ordering=%s"
+        self.request("/related?data_tgp_ordering=%s"
             % quote('-user.occupation.name,-id'))
         self.assert_order(3, 15, 9, 8, 6, 4, 16, 12, 11, 7)
         Spy.assert_ok(self.body, 'ordering', ['-user.occupation.name', '-id'])
 
-        self.request("/related?tg_paginate_no=2&tg_paginate_ordering=%s"#
+        self.request("/related?data_tgp_no=2&data_tgp_ordering=%s"#
             % quote('-user.occupation.name,-id'))
         self.assert_order(2, 14, 13, 10, 5, 1)
         Spy.assert_ok(self.body, 'ordering', ['-user.occupation.name', '-id'])
@@ -1158,9 +1049,21 @@ class TestDatabasePagination(TestPagination):
 
 
 def setup_module():
+    global _sa_dburi, _so_dburi
+    _so_dburi = config.get("sqlobject.dburi", "sqlite:///:memory:")
+    _sa_dburi = config.get("sqlalchemy.dburi", "sqlite:///:memory:")
+    # sqlalchemy setup
+    database.set_db_uri("sqlite:///:memory:", 'sqlalchemy')
+    sqlalchemy_cleanup()
+    bind_metadata()
+
+    # sqlobject setup
+    database.set_db_uri("sqlite:///:memory:", 'sqlobject')
+
     create_tables()
     hub.commit()
 
 def teardown_module():
     drop_tables()
     sqlalchemy_cleanup()
+    config.update({"sqlalchemy.dburi": _sa_dburi, 'sqlobject.dburi': _so_dburi})
