@@ -23,10 +23,10 @@ class Info(controllers.Controller):
 
     icon = "/tg_static/images/info.png"
 
-    [expose(template='turbogears.toolbox.info')]
+    @expose(template='kid:turbogears.toolbox.info')
     def index(self):
         from turbogears.command.info import retrieve_info
-        packages, plugins = retrieve_info()
+        packages, plugins = retrieve_info(with_links=True)
         return dict(packages=packages, plugins=plugins)
 
 
@@ -48,7 +48,7 @@ class WidgetBrowser(controllers.Controller):
     all_descs = None
     icon = "/tg_static/images/widgets.png"
 
-    [expose(template="turbogears.toolbox.widgets")]
+    @expose(template="kid:turbogears.toolbox.widgets")
     def index(self, name=None):
         from turbogears import widgets
         from turbogears.widgets import js_location, Tabber, SyntaxHighlighter
@@ -62,8 +62,7 @@ class WidgetBrowser(controllers.Controller):
             self.all_descs = all_descs
         if name:
             all_descs = {name: all_descs[name]}
-        desclist = all_descs.values()
-        desclist.sort(cmp=lambda x, y: cmp(x.name.lower(), y.name.lower()))
+        desclist = sorted(all_descs.itervalues(), key=lambda x: x.name.lower())
         output = dict(descs=desclist, viewing_one=name != None)
         if name:
             # do not extend desclist!
@@ -143,12 +142,12 @@ class Toolbox(controllers.RootController):
             pairs.append([p[1][-1], None])
         return pairs
 
-    [expose(template="turbogears.toolbox.main")]
+    @expose(template="kid:turbogears.toolbox.main")
     def index(self):
         return dict(toolbox = self.arrange_in_pairs(self.toolbox),
             project = get_project_name())
 
-    [expose()]
+    @expose()
     def noaccess(self):
         return """<h3>No access for %s</h3>
             <p>
