@@ -28,7 +28,7 @@ catwalk.renderModels = function(result)
     var handle = TD({'id':'handle_'+ objectName,'class':'handle'},'\u2195');
     var mouseOver ='if(document.getElementById("handle_'+ objectName +'")) document.getElementById("handle_'+ objectName +'").style.visibility="visible"';
     var mouseOut = 'if(document.getElementById("handle_'+ objectName +'")) document.getElementById("handle_'+ objectName +'").style.visibility="hidden"';
-    models.appendChild( 
+    models.appendChild(
                         LI({'itemID':objectName,'onmouseover':mouseOver,'onmouseout':mouseOut},
                            TABLE({'border':'0','cellspacing':'2'}, TBODY(null, TR(null, handle, model) ) )
                         )
@@ -92,18 +92,18 @@ catwalk.hightlightObject = function(objectName)
 {
   var el = document.getElementById('object_list');
   var children = el.getElementsByTagName('a');
-  for (var i = 0; i < children.length; i++) 
+  for (var i = 0; i < children.length; i++)
   {
     var child = children[i]
-    if(child.getAttribute('id') == 'list_'+ objectName) 
+    if(child.getAttribute('id') == 'list_'+ objectName)
     {
-      child.style.color = '#000'; 
-      child.style.fontWeight= '900'; 
+      child.style.color = '#000';
+      child.style.fontWeight= '900';
     }
     else
     {
-      child.style.color = '#999'; 
-      child.style.fontWeight= ''; 
+      child.style.color = '#999';
+      child.style.fontWeight= '';
     }
   }
 }
@@ -117,7 +117,7 @@ catwalk.renderStructure = function(result)
 {
   if(catwalk.error_result(result)) return;
   for(var i=0;i<result['columns'].length;i++)
-  { 
+  {
     result['columns'][i].objectName = result['objectName'];
     result['columns'][i].reorder = result['reorder'];
   }
@@ -129,8 +129,8 @@ catwalk.renderStructure = function(result)
                   )
               );
   replaceChildNodes('content',s);
-  
-  if(result['reorder']) 
+
+  if(result['reorder'])
   {
     dragsort.makeListSortable(document.getElementById(result['objectName']),setHandle,catwalk.endDrag)
   }
@@ -201,7 +201,7 @@ catwalk.columnLabel = function(col)
 }
 catwalk.renderColumn = function(col)
 {
-  var label = TD({'class':'label'},catwalk.columnLabel(col) ); 
+  var label = TD({'class':'label'},catwalk.columnLabel(col) );
   var fieldType = TD({'class':'coltype'},col.type);
 
   if(col.type == 'SOForeignKey' || col.type == 'SOSingleJoin')
@@ -262,14 +262,14 @@ catwalk.renderColumnsForLabel = function(result)
                                'label':label
                                };
   }
-  var action = "catwalk.selectColumnForLabel(this,"; 
+  var action = "catwalk.selectColumnForLabel(this,";
   action += "'"+ result['objectName'] +"','"+ result['foreignObjectName'] +"','"+ result['foreignKeyName'] +"'";
-  action += ")"; 
+  action += ")";
   var labelChooser = createDOM('SELECT',
                                {'onchange':action},
-                                map(catwalk.renderOptions, options) 
+                                map(catwalk.renderOptions, options)
                               );
-  replaceChildNodes('spanid_'+ result['foreignKeyName'],labelChooser); 
+  replaceChildNodes('spanid_'+ result['foreignKeyName'],labelChooser);
 }
 catwalk.selectColumnForLabel = function(columnList,objectName,foreignObjectName,foreignKeyName)
 {
@@ -351,7 +351,7 @@ catwalk.paging_footer = function(result)
     return DIV({'style':'font-size:10px;margin:6px;'},
                previous_page,
                ' ',
-               'Page ', 
+               'Page ',
                catwalk.page_selector(page_number,total_pages,size,result['objectName']) ,
                '  of '+ total_pages,
                ' ',
@@ -408,7 +408,7 @@ catwalk.renderValues = function(result)
              };
 
   var grid = Widget.grid.render(result['objectName'],data);
-  replaceChildNodes('content', DIV(null, 
+  replaceChildNodes('content', DIV(null,
                                    catwalk.renderObjectMenu(result['objectName']),
                                    grid,
                                    catwalk.paging_footer(result)
@@ -430,7 +430,7 @@ catwalk.retrieveFormAdd = function(objectName)
 }
 catwalk.submiting = function()
 {
-   return false; 
+   return false;
 }
 catwalk.renderFormAdd = function(result)
 {
@@ -489,7 +489,7 @@ catwalk.renderButton = function(btnId,label,action,title,accesskey)
 {
   var props = {'class':'buts',
                'id':btnId,
-               'onclick':action 
+               'onclick':action
               };
   if(Widget.exists(title)) props['title'] = title;
   if(Widget.exists(accesskey)) props['accesskey'] = accesskey;
@@ -514,6 +514,9 @@ catwalk.renderFormColumn = function(column)
     case 'SOIntCol':
       columnValue = catwalk.renderNumericCol(column,'int');
       break;
+    case 'SODecimalCol':
+      columnValue = catwalk.renderNumericCol(column,'float');
+      break;
     case 'SOFloatCol':
       columnValue = catwalk.renderNumericCol(column,'float');
       break;
@@ -553,11 +556,11 @@ catwalk.renderForeignKeyColumn = function(column)
                                                              };
   }
   var sel_id = 'foreign_key_'+ column.join;
-  return TD(null, 
+  return TD(null,
             createDOM('SELECT',
                       {'id':sel_id,'name':column.columnName},
-                      map(catwalk.renderOptions, column.options) 
-                     ) 
+                      map(catwalk.renderOptions, column.options)
+                     )
            );
 }
 
@@ -571,11 +574,11 @@ catwalk.renderEnumCol = function(column)
                          'label':column.options[i]
                         }
   }
-  return TD({'style':'border-bottom:none'}, 
+  return TD({'style':'border-bottom:none'},
             createDOM('SELECT',
                       {'name':column.columnName},
-                      map(catwalk.renderOptions, column.options) 
-                     ) 
+                      map(catwalk.renderOptions, column.options)
+                     )
            );
 }
 catwalk.getValueForField = function(form,fieldName)
@@ -614,8 +617,8 @@ catwalk.retrieveObjectCreate = function(objectName,whereToGo)
 {
   var postVars = catwalk.collectPostVars(document.myform);
   postVars+= '&objectName='+ objectName;
-  var d = postJSONDoc('add',postVars); 
-  
+  var d = postJSONDoc('add',postVars);
+
   d.addCallbacks(notify_success, notify_warning);
   d.addCallback((whereToGo=='save')? catwalk.savedObject:catwalk.retrieveContinueFormAdd);
 }
@@ -642,7 +645,7 @@ catwalk.savedObject = function(result)
     if(catwalk.multipleJoinContext['hostObject']==view['objectName'])
     {
        catwalk.retrieveDisplayObject(view['objectName'],view['id']);
-       return; 
+       return;
     }
     catwalk.renderDisplayObject(result);
 }
@@ -708,7 +711,7 @@ catwalk.checkDisclosedRelations = function(objectName,id)
 {
     for(joinName in catwalk.disclosedRelations)
     {
-        var ctx = catwalk.disclosedRelations[joinName]; 
+        var ctx = catwalk.disclosedRelations[joinName];
         if(!ctx['visible']) continue;
         if(ctx['objectName']!=objectName) continue;
         if(ctx['id']!=id) continue;
@@ -718,7 +721,7 @@ catwalk.checkDisclosedRelations = function(objectName,id)
                               ctx['id'],
                               ctx['joinName'],
                               ctx['joinType'],
-                              ctx['joinObjectName']); 
+                              ctx['joinObjectName']);
     }
 }
 catwalk.retrieveManageRelatedJoins = function(objectName,id,joinName,relatedObjectName)
@@ -730,7 +733,7 @@ catwalk.retrieveManageRelatedJoins = function(objectName,id,joinName,relatedObje
 }
 catwalk.optionsForJoins = function(rows)
 {
-  var options = []; 
+  var options = [];
   for(var i=0;i<rows.length;i++)
   {
     var values='';
@@ -747,11 +750,11 @@ catwalk.renderManageRelatedJoins = function(results)
 {
   if(catwalk.error_result(results)) return;
   var thisJoins = createDOM('SELECT',
-                                {'id':'select_remove_'+ results['join'],'size':'10','style':'width:225px','multiple':'multiple'}, 
-                                map(catwalk.renderOptions, catwalk.optionsForJoins(results['rows']) ) 
-                           ); 
+                                {'id':'select_remove_'+ results['join'],'size':'10','style':'width:225px','multiple':'multiple'},
+                                map(catwalk.renderOptions, catwalk.optionsForJoins(results['rows']) )
+                           );
   var allJoins = createDOM('SELECT',{'id':'select_add_'+ results['join'],'size':'10','style':'width:225px','multiple':'multiple'},
-                                map(catwalk.renderOptions, catwalk.optionsForJoins(results['allJoins']['rows']) ) 
+                                map(catwalk.renderOptions, catwalk.optionsForJoins(results['allJoins']['rows']) )
                            );
   var params = [results['hostObject'],results['id'],results['join'],results['joinType'],results['joinObjectName']];
   var joinsTable = TABLE(null,
@@ -823,7 +826,7 @@ catwalk.retrieveJoins= function(objectName,id,joinName,joinType,joinObjectName)
 catwalk.toggleJoins = function(joinName)
 {
   var e = document.getElementById('grid_'+ joinName);
-  e.style.display= (e.style.display=='')? 'none':''; 
+  e.style.display= (e.style.display=='')? 'none':'';
   var arrow =(e.style.display=='')? 'down':'right';
   var newLink = A({
                    'href':"javascript:catwalk.toggleJoins('"+ joinName +"')",
@@ -844,7 +847,7 @@ catwalk.renderJoins = function(results)
                 results['joinObjectName']
                ];
 
- 
+
   var data = { 'headers':results['headers'],
                'rows':results['rows'],
                'hidden_columns':(Widget.exists(results['hidden_columns']))? results['hidden_columns']:[],
@@ -921,7 +924,7 @@ catwalk.renderManageRelatedControl = function(objectName,id,joinMethodName,other
 catwalk.renderDisplayColumn = function(column)
 {
   var label = SPAN({'class':'previewLabel'},catwalk.columnLabel(column) );
-  var val = column.value.value; 
+  var val = column.value.value;
   if(column.type=='SOSingleJoin')
   {
     if(val=='None'||val=='')
@@ -956,7 +959,7 @@ catwalk.renderDisplayColumn = function(column)
                                   'src':tg_static_directory +'/images/arrow_right.png'} )
                             )
                         ),
-                       label, 
+                       label,
                        '  ('+ catwalk.joinLabel(val) +')  ',
                        edit,
                         DIV({'class':'multiplejoin','id':'grid_'+ column.columnName})
@@ -968,15 +971,15 @@ catwalk.renderDisplayColumn = function(column)
 catwalk.discardSingleJoin = function(objectName,id,join_object_name,join_object_id)
 {
     if(!confirm('Are you sure you want to delete the single join?')) return;
-    var e = loadJSONDoc('remove_single_join?object_name='+ 
-                         objectName+'&id='+ 
-                         id+'&join_object_name='+ 
+    var e = loadJSONDoc('remove_single_join?object_name='+
+                         objectName+'&id='+
+                         id+'&join_object_name='+
                          join_object_name +'&join_object_id='+ join_object_id);
     e.addCallback(catwalk.renderDisplayObject);
 }
 catwalk.joinLabel = function(value)
 {
-  var n = parseInt(value); 
+  var n = parseInt(value);
   if(isNaN(n)) n = 0;
   n+= (n ==1 )? ' Row':' Rows';
   return n
@@ -1004,7 +1007,7 @@ catwalk.renderFormEdit = function(result)
 
   var updateButton = catwalk.renderButton('btnUpdate',
                                     ['Update (',SPAN({style: 'text-decoration: underline'}, 's'),')'],
-                                    "catwalk.retrieveObjectUpdate('"+ objectName +"',"+ result['id'] +")",
+                                    "catwalk.retrieveObjectUpdate('"+ objectName +"','"+ result['id'] +"')",
                                     'Update (s)',
                                      's'
                                     );
@@ -1050,29 +1053,31 @@ catwalk.renderStringColumn = function(column)
                                'value':catwalk.getFieldValue(column),
                                'maxlength':column.length
                               }
-                             ) 
+                             )
              );
   }
 
   return TD(null, createDOM('TEXTAREA',
                              {'name':column.columnName, 'class':'txtarea'},
                               catwalk.getFieldValue(column)
-                           ) 
+                           )
            );
 }
-catwalk.validate = function(field,expectedType)
+catwalk.validate = function(field,expectedType,notNone)
 {
-  switch(expectedType) 
-  {
-    case 'int':
-      if(isNaN(parseInt(field.value))) field.value = field.defaultValue;
-    case 'float':
-      if(isNaN(parseFloat(field.value))) field.value = field.defaultValue;
+  if (field.value || notNone) {
+    switch(expectedType)
+    {
+      case 'int':
+        if(isNaN(parseInt(field.value))) field.value = field.defaultValue;
+      case 'float':
+        if(isNaN(parseFloat(field.value))) field.value = field.defaultValue;
+    }
   }
 }
 catwalk.getFieldValue = function(column)
 {
-  var value = (Widget.exists(column.value))? column.value:''; 
+  var value = (Widget.exists(column.value))? column.value:'';
   if(!Widget.exists(value)) value = (Widget.exists(column['default']))? column['default']:'';
   return value;
 }
@@ -1083,9 +1088,9 @@ catwalk.renderNumericCol = function(column,digitType)
                              'type':'text',
                              'name':column.columnName,
                              'value':catwalk.getFieldValue(column),
-                             'onblur':'catwalk.validate(this,"'+ digitType +'")'
+                             'onblur':'catwalk.validate(this,"'+ digitType +'",' + column.notNone + ')'
                             }
-                           ) 
+                           )
            );
 }
 catwalk.updated = function(results)
@@ -1097,7 +1102,7 @@ catwalk.retrieveObjectUpdate = function(objectName,id)
   var postVars =catwalk.collectPostVars(document.myform);
   postVars+= '&objectName='+ objectName +'&id='+ id;
 
-  var d = postJSONDoc('update',postVars); 
+  var d = postJSONDoc('update',postVars);
   d.addCallbacks(notify_success, notify_warning);
   d.addCallback(catwalk.updated);
 }
@@ -1106,13 +1111,13 @@ function postJSONDoc(url,postVars)
   var req = getXMLHttpRequest();
   req.open('POST',url,true);
   req.setRequestHeader('Content-type','application/x-www-form-urlencoded');
-  var data = postVars; 
+  var data = postVars;
   var d = sendXMLHttpRequest(req,data);
   return d.addCallback(evalJSONRequest);
 }
 catwalk.removedRow = function(result)
 {
-    if(catwalk.history.length < 1) 
+    if(catwalk.history.length < 1)
     {
         catwalk.browse(result['objectName']);
         return;
