@@ -1,18 +1,17 @@
-import os
 import itertools
-import pkg_resources
+import os
 import warnings
-from turbogears import view, startup, config
-from turbogears.util import setlike, to_unicode, copy_if_mutable, \
-    get_package_name
+
+import pkg_resources
+
+from cherrypy import request
+import tgmochikit
+
+from turbogears import config, startup, view 
+from turbogears.util import (setlike, to_unicode, copy_if_mutable,
+    get_package_name, request_available)
 from turbogears.i18n.utils import get_locale
 from turbogears.widgets.meta import MetaWidget, load_kid_template
-from cherrypy.config import configs
-
-try:
-    set
-except NameError: # Python 2.3
-    from sets import Set as set
 
 __all__ = ['load_widgets', 'all_widgets', 'Widget', 'CompoundWidget',
     'WidgetsList', 'register_static_directory', 'static',
@@ -83,8 +82,8 @@ PlainHTML = load_kid_template("""
 class Widget(object):
     """A TurboGears Widget.
 
-   '__init__' and 'update_params' are the only methods you might need to
-   care extending.
+    '__init__' and 'update_params' are the only methods you might need to
+    care extending.
 
     Attributes you should know about:
 
@@ -107,28 +106,27 @@ class Widget(object):
     params_doc: A dictionary containing 'params' names as keys and their
                 docstring as value. For documentation at the widget browser.
 
-   All initialization parameters listed at the class attribute "params" can be
-   defined as class attributes, overriden at __init__ or at display time. They
-   will be treated as special params for the widget, which means:
+    All initialization parameters listed at the class attribute "params" can be
+    defined as class attributes, overriden at __init__ or at display time. They
+    will be treated as special params for the widget, which means:
 
-   1) You can fix default values for them when sublcassing Widget.
+    1) You can fix default values for them when sublcassing Widget.
 
-   2) If passed as **params to the constructor, the will be bound automatically
-      to the widget instance, taking preference over any class attributes
-      previously defined. Mutable attributes (dicts and lists) defined as class
-      attributes are safe to modify as care is taken to copy them so the class
-      attribute remains unchanged.
+    2) If passed as **params to the constructor, the will be bound automatically
+       to the widget instance, taking preference over any class attributes
+       previously defined. Mutable attributes (dicts and lists) defined as class
+       attributes are safe to modify as care is taken to copy them so the class
+       attribute remains unchanged.
 
-   3) They can be further overrided by passing them as keyword args to
-      display(). This will only affect that display() call in a thread-safe
-      way.
+    3) They can be further overrided by passing them as keyword args to
+       display(). This will only affect that display() call in a thread-safe
+       way.
 
-   4) A callable can be passed and it will be called automatically when sending
-      variables to the template. This can be handy to pick up parameters which
-      change in every request or affect many widgets simultaneously.
+    4) A callable can be passed and it will be called automatically when sending
+       variables to the template. This can be handy to pick up parameters which
+       change in every request or affect many widgets simultaneously.
 
     """
-
     __metaclass__ = MetaWidget
 
     name = "widget"
@@ -179,7 +177,7 @@ class Widget(object):
         if template:
             self.template_c, self.template = load_kid_template(template)
 
-        if default:
+        if default is not None:
             self.default = default
 
         # logic for managing the params attribute
@@ -189,15 +187,12 @@ class Widget(object):
                 setattr(self, param, copy_if_mutable(params.pop(param)))
 
             else:
-                if hasattr(self, param):
-                    # make sure we don't alter mutable class attributes
-                    value = copy_if_mutable(getattr(self.__class__, param),
-                                            True)
-                    if value[1]:
-                        # re-set it only if mutable
-                        setattr(self, param, value[0])
-                else:
-                    setattr(self, param, None)
+                # make sure we don't alter mutable class attributes
+                value, mutable = copy_if_mutable(
+                    getattr(self.__class__, param), True)
+                if mutable:
+                    # re-set it only if mutable
+                    setattr(self, param, value)
 
         for unused in params.iterkeys():
             warnings.warn('keyword argument "%s" is unused at %r instance' % (
@@ -230,9 +225,6 @@ class Widget(object):
     # there is rarely the need to override or extend them if inheritting
     # directly or indirectly from Widget.
 
-    # update_data has been deprecated
-    update_data = update_params
-
     def __call__(self, *args, **params):
         """Delegate to display. Used as an alias to avoid tiresome typing."""
         return self.display(*args, **params)
@@ -255,7 +247,6 @@ class Widget(object):
                   template.
 
         """
-
         if not getattr(self, 'template_c', False):
             warnings.warn("Widget instance '%r' has no template defined" % self)
             return None
@@ -280,19 +271,36 @@ class Widget(object):
             params['name'] = self.name
         params['value'] = to_unicode(self.adjust_value(value, **params))
         self.update_params(params)
-        # update_data has been deprecated
-        self.update_data(params)
 
         try:
             transform = view.engines['kid'].transform
-
         except (KeyError, AttributeError):
             # this can happen if you render a widget before application startup
             # when view.load_engines() has not yet been called
             raise RuntimeError("Trying to render a widget,"
                 " but Kid templating engine is not yet loaded.")
 
-        return transform(params, self.template_c)
+        # If the page template is Genshi, we keep track of the nesting level,
+        # because Genshi cannot display Kid's ElementTree elements directly.
+        if request_available() and request.tg_template_enginename == 'genshi':
+            display_level = getattr(request, 'tg_widgets_display_level', 0)
+            request.tg_widgets_display_level = display_level + 1
+        else:
+            display_level = None
+
+        try:
+            output = transform(params, self.template_c)
+            if display_level == 0:
+                # On the topmost level, we create a Genshi markup stream
+                # from Kid's ElementTree element to make Genshi really happy.
+                # This automatism makes wrapping widget output with ET(...)
+                # calls in Genshi page templates unnecessary.
+                output = view.genshi_et(output)
+        finally:
+            if display_level is not None:
+                request.tg_widgets_display_level = display_level
+
+        return output
 
     def render(self, value=None, format="html", **params):
         """Exactly the same as display() but return serialized output instead.
@@ -302,8 +310,8 @@ class Widget(object):
 
         """
         elem = self.display(value, **params)
-        t = PlainHTML(elements=elem)
-        return t.serialize(output=format, fragment=True)
+        template = PlainHTML(elements=elem)
+        return template.serialize(output=format, fragment=True)
 
     def retrieve_javascript(self):
         """Return the needed JavaScript ressources.
@@ -465,14 +473,16 @@ class MetaWidgetsList(type):
     """
     def __new__(meta, class_name, bases, class_dict):
         declared_widgets = []
+        for base in bases:
+            declared_widgets.extend(getattr(base, 'declared_widgets', []))
         for name, value in class_dict.items():
             if isinstance(value, Widget):
                 if not value.is_named:
                     value.name = name
                 declared_widgets.append(value)
-                del class_dict[name]
-        declared_widgets.sort(lambda a, b: cmp(a._declaration_counter,
-                                               b._declaration_counter))
+                # we keep the widget accessible as attribute here,
+                # i.e. we don't del class_dict[name] as we did before
+        declared_widgets.sort(key=lambda w: w._declaration_counter)
         cls = type.__new__(meta, class_name, bases, class_dict)
         cls.declared_widgets = declared_widgets
         return cls
@@ -488,13 +498,13 @@ class WidgetsList(list):
     __metaclass__ = MetaWidgetsList
 
     def __init__(self, *args):
+        super(WidgetsList, self).__init__(self.declared_widgets)
         if args:
-            if len(args) > 1:
-                args = [args]
-            elif isinstance(args[0], Widget):
-                args = [[args[0]]]
-        super(WidgetsList, self).__init__(*args)
-        self.extend(self.declared_widgets)
+            if len(args) == 1:
+                args = args[0]
+                if isinstance(args, Widget):
+                    args = [args]
+            self.extend(args)
         if not self:
             warnings.warn("You have declared an empty WidgetsList")
 
@@ -503,12 +513,12 @@ class WidgetsList(list):
     # contained widgets get picked up by the templates:
 
     def retrieve_javascript(self):
-        return itertools.chain(*[w.retrieve_javascript() for w in self
-            if callable(getattr(w, 'retrieve_javascript', None))])
+        return itertools.chain(*(w.retrieve_javascript() for w in self
+            if callable(getattr(w, 'retrieve_javascript', None))))
 
     def retrieve_css(self):
-        return itertools.chain(*[w.retrieve_css() for w in self
-            if callable(getattr(w, 'retrieve_css', None))])
+        return itertools.chain(*(w.retrieve_css() for w in self
+            if callable(getattr(w, 'retrieve_css', None))))
 
 
 #############################################################################
@@ -524,7 +534,7 @@ def register_static_directory(modulename, directory):
     """
     directory = os.path.abspath(directory)
     config.update({'/tg_widgets/%s' % modulename: {
-        'static_filter.on': True,
+        'static_filter.on' : True,
         'static_filter.dir': directory
     }})
 
@@ -550,7 +560,9 @@ class Resource(Widget):
     or inline source to include at the template the widget is displayed.
 
     """
-    pass
+    order = 0
+    params = ['order']
+    params_doc = {'order': "JS and CSS are sorted in this 'order' at render time"}
 
 
 class Link(Resource):
@@ -574,7 +586,7 @@ class Link(Resource):
 
 
 class CSSLink(Link):
-    """A CSS links."""
+    """A CSS link."""
 
     template = """
     <link rel="stylesheet"
@@ -612,52 +624,57 @@ class JSLink(Link):
         location = kw.pop('location', None)
         super(JSLink, self).__init__(*args, **kw)
         if location:
-            if location not in js_location:
-                raise ValueError, "JSLink location should be in %s" % js_location
+            #if location not in js_location:
+            #    raise ValueError, "JSLink location should be in %s" % js_location
             self.location = location
 
     retrieve_javascript = set_with_self
 
 
-class MochiKitLink(JSLink):
-    """Specialized JSLink to allow for suppression of MochiKit inclusion.
+class TGMochiKit(JSLink):
+    """This JSLink includes MochkKit by means of the tgMochiKit project.
 
-    The rationale for this is that MochiKit in version 1.3.1 has some issues
-    that are are remedied by 1.4, but the latter is unfortunately unreleased.
+    Depending on three config options, you can decide
 
-    To remedy this, you can include MochiKit 1.4 by yourself (using e.g.
-    tgMochiKit) - but this will conflict with the contained version which
-    is made use of in several TG widgets.
+    ``tg_mochikit.version`` -- ``"1.3.1"``
+        Which version of MochiKit should be used.
+    ``tg_mochikit.packed`` -- ``False``
+        Whether to deliver the MochiKit JS code packed or unpacked.
+    ``tg_mochikit.xhtml`` -- ``False``
+        Whether to be XHTML-compliant or not when ``tg_mochikit.packed`` is
+        ``False``, i.e. selects whether the ``turbogears.mochikit`` widget
+        should include all of MochiKit's submodules as separate JavaScript
+        resources or just the main ``MochiKit.js`` file.
 
-    By setting the configuration option
+    For more explanation of the options, see the tgMochiKit documentation at
 
-      tg.mochikit_suppress = True
-
-    the incusion of MK via built-in widgets will be suppressed. Of course then
-    one has to include the proper version explicitly, or via tg.include_widgets.
-
-    If 'turbogears.mochikit' is part of tg.include_widgets, a warning is issued.
+        http://docs.turbogears.org/tgMochiKit
 
     """
 
-    template = """
-    <span xmlns:py="http://purl.org/kid/ns#" py:strip="True">
-        <script py:if="not suppress" type="text/javascript" src="$link"/>
-        <script py:if="suppress" type="text/javascript"></script>
-    </span>
+    template = """<script xmlns:py="http://purl.org/kid/ns#"
+    py:for="js in javascripts" py:replace="js.display()"
+    />
     """
 
-    def update_params(self, p):
-        super(MochiKitLink, self).update_params(p)
-        suppress = config.get('tg.mochikit_suppress', False)
-        p['suppress'] = suppress
-        if suppress and 'turbogears.mochikit' in config.get('tg.include_widgets', []):
-            warnings.warn("""\
+    def retrieve_javascript(self):
+        tgmochikit.init(register_static_directory, config)
+        if config.get('tg.mochikit_suppress', False):
+            if 'turbogears.mochikit' in config.get('tg.include_widgets', []):
+                warnings.warn("""\
 tg.mochikit_suppress == True, but 'turbogears.mochikit' is to be included via
 'tg.include_widgets'. This is a contradiction, and mochikit_suppress overrides
 the inclusion to prevent subtle errors due to version mixing of MochiKit.""")
+            return []
+        javascripts = [JSLink("tgmochikit", path)
+            for path in tgmochikit.get_paths()]
+        return javascripts
 
-mochikit = MochiKitLink("turbogears", "js/MochiKit.js")
+    def update_params(self, d):
+        super(TGMochiKit, self).update_params(d)
+        d['javascripts'] = self.retrieve_javascript()
+
+mochikit = TGMochiKit("tgmochikit")
 
 
 class JSI18NWidget(Widget):
@@ -680,7 +697,6 @@ class JSI18NWidget(Widget):
     in your application configuration.
 
     """
-
     params = ['locale_catalog_providers']
 
     locale_catalog_providers = []
@@ -726,6 +742,7 @@ class JSI18NWidget(Widget):
                         js.append(link)
                         break
                 return js
+
         self.locale_catalog_providers.append(PackageProvider())
 
     def retrieve_javascript(self):
@@ -741,14 +758,12 @@ class JSI18NWidget(Widget):
         return js
 
     def linked_file_exists(self, widget):
-        key = '/tg_widgets/%s' % widget.mod
-        wp = configs.get(key)
-        if wp:
-            dir = wp.get('static_filter.dir')
-            if dir:
-                path = os.path.join(dir, widget.name)
-                return os.path.exists(path)
-        return False
+        static_dir = config.get('static_filter.dir',
+            path='/tg_widgets/%s' % widget.mod)
+        if static_dir:
+            return os.path.exists(os.path.join(static_dir, widget.name))
+        else:
+            return False
 
 jsi18nwidget = JSI18NWidget()
 
