@@ -4,7 +4,8 @@ from turbojson import jsonify
 __all__ = ["RPC"]
 
 class RPC(Widget):
-    """ RPC base widget """
+    """RPC base widget."""
+
     params = ["action", "update", "data", "on_success", "on_failure", "on_complete",
                      "before", "after", "loading", "loaded", "confirm"]
 
